'use strict';
/**
 * Building utility to run multiple npm commands
 */
const exec = require('child_process').exec,
  async = require('async'),
  os = require('os'),
  cpuCount = os.cpus().length,
  npmScripts = process.argv.splice(2);

let pIdx = npmScripts.indexOf('--series'),
  isSeries = (pIdx !== -1);

if (isSeries) {
  npmScripts.splice(pIdx, 1);
}

let calls = [];
npmScripts.forEach((name) => {
  calls.push((done) => {
    console.log('RUN: ' + name);
    let child = exec('npm run -s ' + name, (err, stdout, stderr) => {
      console.log('DONE: ' + name);
      if (err) {
        if(!isSeries) {
          console.error(stderr);
          process.exit(1);
        }
        return done(stderr);
      }
      done();
    });
    child.stdout.pipe(process.stdout);
    child.stderr.pipe(process.stderr);
  });
});

function onCompleted(err) {
  if (err) {
    console.error(err);
    return process.exit(1);
  }
  process.exit(0);
}
if(isSeries) {
  async.series(calls, onCompleted);
} else {
  async.parallelLimit(calls, cpuCount, onCompleted);
}
