'use strict';
/**
 * Performs save npm shrinkwraps.
 */
const exec = require('child_process').exec,
  async = require('async'),
  fs = require('fs');

let calls = [];

/* perform npm shrinkwrap */
calls.push((done) => {
  console.log('Running npm shrinkwrap');
  let args = process.argv.splice(2);
  exec('npm shrinkwrap ' + args.join(' '), (err, stdout, stderr) => {
    if (err) {
      return done(err);
    }
    console.log('Result');
    console.log(stdout);
    if (stderr && stderr.trim().length > 0) {
      console.log('Warnings');
      console.log(stderr);
    }
    done();
  });
});

/* check the npm-shrinkwrap.json */
calls.push((done) => {
  let content;
  try {
    content = fs.readFileSync('./npm-shrinkwrap.json', {encoding: 'utf8'});
  } catch (e) {
    if (e && e.code === 'ENOTFOUND') {
      return done();  // no file.
    }
    return done(e);
  }
  content = content.replace(/http:/g, 'https:');
  try {
    fs.writeFileSync('./npm-shrinkwrap.json', content, {encoding: 'utf8'});
  } catch (e) {
    return done(e);
  }
  done();
});

async.series(calls, (e) => {
  if (e) {
    console.error(e);
    return process.exit(1);
  }
  console.log('Safe shrinkwrap completed');
});
