'use strict';
const tredux = require('tredux'),
  api = require('api');

export const TYPE = {
  SET_TITLE: 'core.ui.title',
  IS_BOOTED: 'core.ui.isBooted',
  SHOW_MENU: "core.ui.showMenu",
  HIDE_MENU: 'core.ui.hideMenu',
  MINIMIZE_MENU: 'core.ui.miniMenu',
  URL_CHANGE: 'core.url.change',
  IS_LOADING: 'LOADING'
};

export function setTitle(title) {
  return {
    type: TYPE.SET_TITLE,
    payload: {
      title
    }
  }
}

export function boot() {
  return {
    type: TYPE.IS_BOOTED
  }
}

export function changePath(url, payload) {
  return {
    type: TYPE.URL_CHANGE,
    payload: {
      url,
      payload: payload || {}
    }
  }
}

export function toggleMenu() {
  const state = tredux.getState()['core.ui'];
  if (state.menuOpened && !state.menuMinimized) {
    return hideMenu();
  }
  return showMenu();
}

export function showMenu() {
  return {
    type: TYPE.SHOW_MENU
  }
}

export function hideMenu() {
  return {
    type: TYPE.HIDE_MENU
  }
}

export function minimizeMenu(subMenu) {
  return {
    type: TYPE.MINIMIZE_MENU,
    payload: {
      active: true,
      subMenu
    }
  }
}

export function maximizeMenu() {
  return {
    type: TYPE.MINIMIZE_MENU,
    payload: {
      active: false
    }
  }
}

export function showLoading(blocking) {
  return {
    type: TYPE.IS_LOADING,
    payload: {
      active: true,
      block: !!blocking
    }
  }
}

export function hideLoading() {
  return {
    type: TYPE.IS_LOADING,
    payload: {
      active: false
    }
  }
}

tredux.addActions('core.ui', module.exports);
