/**
 * Created by zsoltmakai on 9/14/2016.
 */
'use strict';
const React = require('react'),
  tredux = require('tredux'),
  router = require('router');

import Avatar from 'material-ui/Avatar';
import FontIcon from 'material-ui/FontIcon';

export default class AccountAvatar extends React.Component {

  renderIcon() {

    const localStyle = {
      fontSize: this.props.size * 0.6
    };
    let color = this.props.color,
      icon = this.props.icon;

    return (
      <FontIcon
        color={color}
        style={localStyle}
        className="material-icons">
        {icon}
      </FontIcon>
    )
  }

  render() {
    let image;
    if (typeof this.props.image === 'string' && this.props.image) {
      image = this.props.image;
    } else if (typeof this.props.image === 'object' && this.props.image && typeof this.props.image.url === 'string' && this.props.image.url) {
      image = this.props.image.url;
    }
    let avatarStyle = {};
    if (image) {
      if (this.props.onImage) {
        image = this.props.onImage(image);
      }
      if (image) {
        avatarStyle.backgroundImage = `url("${image}")`;
      }
    }
    let textStyle = {};
    if (this.props.textColor) {
      textStyle.color = this.props.textColor;
    }
    return (
      <div className={classNames("avatar-container", this.props.className)} style={this.props.style}>
        <Avatar
          className="account-avatar"
          style={avatarStyle}
          size={this.props.size}
          icon={image ? undefined : this.renderIcon()}/>
        <div
          className="avatar-text"
          onClick={this.props.onClick}
          style={textStyle}>
          {this.props.title && <span className="avatar-text-title">{this.props.title.toUpperCase()}</span>}
          {this.props.subTitle && <span className="avatar-text-sub">{this.props.subTitle}</span>}
        </div>
      </div>
    )
  }
}

AccountAvatar.PropTypes = {
  title: React.PropTypes.string,
  subTitle: React.PropTypes.string,
  onImage: React.PropTypes.func,
  icon: React.PropTypes.string,
  image: React.PropTypes.string,
  color: React.PropTypes.string
};

AccountAvatar.defaultProps = {
  icon: 'person',
  color: 'BDBDBD',
  size: 40

};
