'use strict';

const React = require('react'),
  tredux = require('tredux'),
  router = require('router'),
  config = require('config'),
  AppMenuItem = require('./AppMenuItem'),
  AppMenuMargin = require('./AppMenuMargin');
import Paper from 'material-ui/Paper';

class AppMenu extends React.Component {

  constructor(props) {
    super(props);
    this._submenuCalled = false;
    this._submenuTimer = null;
  }

  componentWillUnmount() {
    clearTimeout(this._submenuTimer);
  }

  componentWillReceiveProps(nextProps) {
    this.getChildren(nextProps, true);
  }

  handleSubmenu(type, name, force) {
    if (typeof name === 'string') {
      if (name.charAt(name.length - 1) === '/') name = name.substr(0, name.length - 1);
      if (name.charAt(0) === '/') name = name.substr(1);
    }
    clearTimeout(this._submenuTimer);
    if (force === true) {
      if (type === 'show') {
        this.props.onSubmenuShow(name);
      } else {
        this.props.onSubmenuHide();
      }
      this._submenuCalled = true;
      this._submenuTimer = setTimeout(() => {
        this._submenuCalled = false;
      }, 50);
      return;
    }
    if (this._submenuCalled) return;
    const ui = tredux.getState()['core.ui'];
    this._submenuCalled = true;
    this._submenuTimer = setTimeout(() => {
      this._submenuCalled = false;
      if (type === 'show') {
        this.props.onSubmenuShow(name);
      } else {
        this.props.onSubmenuHide();
      }
    }, 10);
  }

  /*
  * Checks if we should toggle the minimized submenu as follows:
  * CURRENT = /applications
  * PATH = /applications => match = true.
  *
  * CURRENT = /applications/dashboard
  * PATH = /applications/ => MATCH = true (since we want to open for all applications/*
  *
  * */
  isSubmenuActive(path) {
    if (!path) return false;
    let current = router.getCurrentPath();
    if (path.charAt(0) !== '/') path = '/' + path;
    if (current.charAt(0) !== '/') current = '/' + current;
    if (current.charAt(current.length - 1) === '/') current = current.substr(0, current.length - 1);
    if (current.indexOf(path) === 0) return true;
    return false;
  }

  getChildren(props, force) {
    let menuItems = [],
      children = [];
    if (!props.onSubmenuShow || !props.onSubmenuHide) return props.children;
    if (!(props.children instanceof Array)) {
      children = [props.children];
    } else {
      children = props.children.map(a => a);
    }
    let activeMenu = null;
    for (let i = 0; i < children.length; i++) {
      let itm = children[i];
      if (typeof itm !== 'object' || !itm) continue;
      let newProps = {};
      if (typeof itm.props.path === 'string' && router.getBasePath() === itm.props.path.toLowerCase()) {
        activeMenu = itm;
      } else if (itm.props.active === true && !activeMenu) {
        activeMenu = itm;
      }
      let _oldClick = itm.props.onClick;
      newProps.onClick = (e) => {
        if (itm.props.subMenu && this.isSubmenuActive(itm.props.subMenu)) {
          this.handleSubmenu('show', itm.props.subMenu);
        } else {
          this.handleSubmenu('hide')
        }
        if (_oldClick) {
          return _oldClick(e);
        }
        return false;
      };
      let itemObj = React.cloneElement(itm, newProps);
      menuItems.push(itemObj);
    }
    if (force) {
      if (activeMenu && activeMenu.props && activeMenu.props.subMenu && this.isSubmenuActive(activeMenu.props.subMenu)) {
        this.handleSubmenu('show', activeMenu.props.subMenu, true);
      } else {
        this.handleSubmenu('hide', null, true);
      }
    }
    return menuItems;
  }

  render() {
    const {ui} = this.props,
      menuItems = this.getChildren(this.props);
    return (
      <Paper
        className={classNames({
          "menu-container": true,
          "menu-opened": ui.menuOpened,
          "menu-minimized": ui.menuMinimized
        })}
        style={this.props.style}
        id={this.props.id}
        zDepth={1} rounded={false}>
        {menuItems}
        {this.renderFooter()}
      </Paper>
    )
  }

  renderFooter() {
    const {ui} = this.props;
    if (ui.menuOpened && !ui.menuMinimized) {
      let footer = this.props.footer;
      if (!footer) return;
      return (
        <div className="menu-footer">
          {footer}
        </div>
      )
    }
    let footerIcon = this.props.footerIcon;
    if (!footerIcon) return;
    return (
      <AppMenuMargin.Footer>
        {footerIcon}
      </AppMenuMargin.Footer>
    );
  }
}

module.exports = tredux.connect((state) => {
  return {
    ui: state['core.ui']
  }
}, AppMenu);
module.exports.Item = AppMenuItem;
module.exports.Header = AppMenuMargin.Header;
module.exports.Footer = AppMenuMargin.Footer;
AppMenu.PropTypes = {
  id: React.PropTypes.string,
  onSubmenuShow: React.PropTypes.func,
  onSubmenuHide: React.PropTypes.func,
  footer: React.PropTypes.any,
  footerIcon: React.PropTypes.any
};
