/**
 * Created by zsoltmakai on 9/14/2016.
 */
'use strict';
const React = require('react');

import Toggle from 'material-ui/Toggle';

export default class AppToggle extends React.Component {
  constructor(props) {
    super(props);
    this._defaultToggled = props.defaultToggled;
    this.state = {active: props.defaultToggled};
    if (typeof props.toggled === 'boolean') {
      this.state.active = props.toggled;
    }
    if (typeof this.state.active === 'undefined') this.state.active = false;
    this.handleToggled = this.handleToggled.bind(this);
  }

  componentWillReceiveProps(nextProps) {
    if (typeof nextProps.toggled === 'boolean') {
      this.setState({
        active: nextProps.toggled
      });
    }
  }

  getValue() {
    return this.state.active;
  }

  setValue(active) {
    this.setState({
      active
    })
  }

  handleToggled(e, isActive) {
    this.setState({active: !this.state.active}, () => {
      this.props.onToggle && this.props.onToggle(isActive);
    });
  }

  render() {
    if (this.props.switch === false) {
      return this.renderSwitch();
    }
    return this.renderToggle();
  }

  renderToggle() {
    let props = Object.assign({}, this.props);
    props.toggled = this.state.active;
    delete props.defaultToggled;
    let errText;
    if (typeof props.errorText === 'object' && props.errorText) {
      errText = props.errorText[props.name];
    } else if (typeof props.errorText === 'string') {
      errText = props.errorText;
    }
    delete props.errorText;
    return (
      <div>
        <Toggle
          {...props}
          className={classNames("app-toggle", this.props.className)}
          ref="toggle"
          labelPosition="right"
          onToggle={this.handleToggled}
        />
        {errText && (
          <p className="tip tip-error no-border" style={{marginLeft: 46}}>{errText}</p>
        )}
      </div>
    )
  }

  renderSwitch() {
    let switchClass = '',
      localStyle,
      activeStyle = {},
      defaultClass;
    if (this.props.multipleLabel === true) {
      if (this.props.switch === false) {
        if (this.state.active) {
          switchClass = 'non-switch1';
          activeStyle = {color: '#000000'};
        } else {
          switchClass = 'non-switch2';
          activeStyle = {color: '#C8C8C8'};
        }
      }
      localStyle = !this.state.active ? {color: '#3da0e3'} : {color: this.props.color || '#C8C8C8'};
      // activeStyle = this.state.active ? {color: '#000000'} : {color: '#C8C8C8'};
      defaultClass = classNames({'toggle-container': true});
    } else {
      if (this.props.switch === false) {
        if (this.state.active) {
          switchClass = 'non-switch2';
          activeStyle = {color: '#3da0e3'};
        } else {
          switchClass = 'non-switch1';
          activeStyle = {color: '#C8C8C8'};
        }
      }
      // activeStyle = this.state.active ? {color: '#3dae0e3'} : {color: this.props.color || '#3dae0e3'};
      defaultClass = classNames({'toggle-container negative-left-16': true});
    }
    let errText;
    if (typeof this.props.errorText === 'object' && this.props.errorText) {
      errText = this.props.errorText[props.name];
    } else if (typeof this.props.errorText === 'string') {
      errText = this.props.errorText;
    }
    return (
      <div className={`${defaultClass} ${switchClass}`} style={this.props.style}>

        {this.props.leftText && <span
          className="toggle-left-text"
          style={localStyle}>
          {this.props.leftText}
        </span>}
        <Toggle
          name={this.props.name}
          toggled={this.props.toggled}
          defaultToggled={this.props.defaultToggled}
          style={{width: 38, marginTop: 2}}
          onToggle={this.handleToggled}
          thumbStyle={this.props.thumbStyle}
        />
        {this.props.rightText && <span
          className="toggle-right-text"
          style={activeStyle}>
          {this.props.rightText}
        </span>}
        {errText && (
          <p className="tip tip-error no-border" style={{marginLeft: 46}}>{errText}</p>
        )}
      </div>
    )
  }
}

AppToggle.PropTypes = {
  defaultToggled: React.PropTypes.bool,
  leftText: React.PropTypes.string,
  rightText: React.PropTypes.string,
  switch: React.PropTypes.bool,
  multipleLabel: React.PropTypes.bool,
  onToggle: React.PropTypes.func,
  color: React.PropTypes.string
};
