'use strict';
const React = require('react'),
  router = require('router');
import FlatButton from 'material-ui/FlatButton';
import RaisedButton from 'material-ui/RaisedButton';
import CopyToClipboard from 'react-copy-to-clipboard';

const theme = require('theme'),
  util = require('../../../lib/util');

/*
 * PARAM FOR COPYING:
 *   copy=text to copy
 *   onCopy=fn on copied.
 * */
export default class Button extends React.Component {

  render() {
    if (!this.props.copy) {
      return this.renderButton();
    }
    return (
      <CopyToClipboard
        text={this.props.copy}
        onCopy={this.props.onCopy}
      >
        {this.renderButton()}
      </CopyToClipboard>
    )
  }

  renderButton() {
    let isOutline = false,
      props = Object.assign({}, this.props);
    if (props.outline) {
      delete props.outline;
      isOutline = true;
    }
    if (props.href) {
      let url = props.href;

      props.onClick = () => {
        if (url.indexOf('http') === 0 && url.indexOf(window.location.origin) === -1) {
          window.open(url, 'Downloading');
          return;
        }
        if (props['data-redirect']) {
          window.location.href = url;
          return;
        }
        router.navigate(url);
        this.props.onClick && this.props.onClick();
      }
    }
    let bgColor = (isOutline ? theme.palette.accent2Color || '#ffffff' : theme.palette.accent1Color || '#000000'),
      hoverColor = theme.palette.accent1HoverColor || util.lighten(bgColor, 15) || '#4B4B4B',
      rippleColor = theme.palette.rippleColor || hoverColor,
      isFlat = !!props.flat;
    if (isFlat) {
      bgColor = undefined;
      hoverColor = undefined;
    }
    if (this.props.primary && theme.palette.primary1Color) {
      bgColor = theme.palette.primary1Color;
      hoverColor = util.darken(bgColor, 10);
      delete props.primary;
    }
    if (this.props.backgroundColor) {
      bgColor = this.props.backgroundColor;
    }
    delete props.flat;
    delete props.href;

    return (
      <FlatButton
        {...props}
        className={classNames(props.className, {
          'app-button': !isFlat,
          'button-outline': isOutline,
          'button-flat': isFlat
        })}
        backgroundColor={bgColor}
        hoverColor={hoverColor}
        rippleColor={rippleColor}
      />
    )
  }
}

Button.PropTypes = {
  copy: React.PropTypes.string,
  onCopy: React.PropTypes.func,
  onClick: React.PropTypes.func,
  outline: React.PropTypes.bool,
  href: React.PropTypes.string,
  flat: React.PropTypes.bool
};

/* PROPS:
 * align: left/right/center */
Button.Group = class ButtonGroup extends React.Component {

  render() {
    let flexDirection = 'flex-start',
      align = this.props.align || 'left';
    if (align === 'right') {
      flexDirection = 'flex-end';
    }
    return (
      <div style={{justifyContent: flexDirection}}
           className={classNames("app-button-group", "app-button-group-" + align, this.props.className)}>
        {this.props.children}
      </div>
    )
  }
};

Button.Group.PropTypes = {
  align: React.PropTypes.string
};
