'use strict';

const React = require('react');

import ArrowDropDown from 'material-ui/svg-icons/navigation/arrow-drop-down';
import Button from 'core/components/Button';
import IconMenu from 'material-ui/IconMenu';
import IconButton from 'material-ui/IconButton/IconButton';
import MenuItem from 'material-ui/MenuItem';

class ButtonDropdown extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      open: props.open || false
    };
  }

  render() {
    let buttonProps = Object.assign({}, this.props),
      items = [],
      children = (this.props.children instanceof Array ? this.props.children : [this.props.children]);
    for (let i = 0, len = children.length; i < len; i++) {
      let c = children[i];
      if (typeof c !== 'object' || !c) continue;
      if (c.type === MenuItem) {
        let opt = {
          ...c.props,
          key: i
        };
        let item = React.cloneElement(c, opt);
        items.push(item);
      }
    }
    delete buttonProps.children;
    return (
      <div className="app-button-dropdown">
        <Button
          {...buttonProps}
          onClick={(e) => this.setState({
            open: true
          })}
        />
        {items.length > 0 && (
          <IconMenu
            className="app-dropdown"
            animated={true}
            desktop={true}
            onRequestChange={() => this.setState({open: false})}
            open={this.state.open}
            useLayerForClickAway={true}
            onClick={() => this.setState({open: true})}
            iconButtonElement={<IconButton style={{padding: 0, width: 36, height: 36}}><ArrowDropDown/></IconButton>}
            anchorOrigin={{horizontal: 'right', vertical: 'top'}}
            targetOrigin={{horizontal: 'right', vertical: 'top'}}
            ref={(c) => this.dropdown = c}
          >
            {items}
          </IconMenu>
        )}
      </div>
    )
  }
}

ButtonDropdown.Item = MenuItem;

module.exports = ButtonDropdown;
