'use strict';
const React = require('react'),
  router = require('router');

import Button from './Button';
import FileInput from './FileInput';

let id = 0;
export default class ButtonFileSelect extends React.Component {

  constructor(props) {
    super(props);
    id++;
    this.handleChange = this.handleChange.bind(this);
  }

  cancel() {
    let $input = $(`#file-reader-${id}`);
    if ($input.length == 0) return;
    $input.val("");
  }

  handleChange(e, res) {
    let fileObj;
    try {
      fileObj = res[0][1];
    } catch (e) {
      return false;
    }
    if (!fileObj) return;
    let ext = [];
    if (this.props.extension instanceof Array) ext = this.props.extension;
    if (typeof this.props.extension === 'string') ext = this.props.extension.split(',');
    let reader = new FileReader();
    if (this.props.buffer === true) {
      reader.readAsArrayBuffer(fileObj);
    } else {
      reader.readAsBinaryString(fileObj);
    }
    let isValid = false;
    if (ext.length === 0) isValid = true;
    let fileExt = fileObj.name.split('.').pop().toLowerCase();
    for (let i = 0; i < ext.length; i++) {
      if (ext[i].toLowerCase() === fileExt) {
        isValid = true;
      }
    }
    if (!isValid) {
      return this.props.onError && this.props.onError('The selected file is not supported');
    }

    reader.onloadend = () => {
      let result = reader.result;
      this.props.onSelect && this.props.onSelect(result, fileExt, fileObj);
    };
    reader.onerror = () => {
      this.props.onError && this.props.onError('An error occurred while opening your file');
    }
  }

  render() {
    let props = Object.assign({}, this.props);
    delete props.extension;
    delete props.buffer;
    return (
      <FileInput id={`file-reader-${id}`} onChange={this.handleChange}>
        <Button
          {...props}
        />
      </FileInput>
    )
  }
}

ButtonFileSelect.PropTypes = {
  extension: React.PropTypes.any,
  onSelect: React.PropTypes.string,
  onError: React.PropTypes.func
};
