'use strict';

const React = require('react');
import * as MuiCheckbox from 'material-ui/Checkbox';
import CheckboxGroup from './CheckboxGroup';

export default class Checkbox extends React.Component {

  constructor(props) {
    super(props);
    this._defaultChecked = props.defaultChecked;
    this.handleChange = this.handleChange.bind(this);
  }

  getValue() {
    return this.checkbox && this.checkbox.state.switched;
  }

  handleChange(e, val) {
    if (this._timer) {
      clearTimeout(this._timer);
    }
    this._timer = setTimeout(() => {
      this.props.onChange && this.props.onChange(val);
      this._timer = null;
    }, 10);
  }

  componentWillUnmount() {
    if (this._timer) clearTimeout(this._timer);
  }

  render() {
    let props = Object.assign({
      style: {}
    }, this.props);
    let isInline = !!props.inline;
    props.defaultChecked = this._defaultChecked;
    delete props.inline;
    delete props.onChecked;
    delete props.onChange;
    return (
      <MuiCheckbox.default
        {...props}
        ref={(c) => this.checkbox = c}
        className={classNames("app-checkbox", {
          'display-inline': isInline
        }, props.className)}
        onCheck={this.handleChange}
      />
    );
  }
}
Checkbox.PropTypes = {
  onChange: React.PropTypes.func,
  inline: React.PropTypes.any,
  onChecked: React.PropTypes.func
};

Checkbox.Group = CheckboxGroup;
