'use strict';
/**
 *
 */
const React = require('react');
import Checkbox from './Checkbox';
import MoreVertIcon from 'material-ui/svg-icons/navigation/more-vert';
import ExpandMoreIcon from 'material-ui/svg-icons/navigation/expand-more';
import ExpandLessIcon from 'material-ui/svg-icons/navigation/expand-less';
import IconButton from 'material-ui/IconButton/IconButton';

export default class CheckboxGroup extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      toggled: props.toggled || false,
      checked: props.checked || false
    };
    this.handleTitleClick = this.handleTitleClick.bind(this);
  }

  isChecked() {
    return this.state.checked;
  }

  componentWillReceiveProps(props) {
    let upd = {};
    if(typeof props.toggled !== 'undefined') {
      upd.toggled = props.toggled;
    }
    if(typeof props.checked !== 'undefined') {
      upd.checked = props.checked;
    }
    this.setState(upd);
  }

  handleTitleClick() {
    this.handleCheckbox(!this.state.checked);
  }

  handleCheckbox(checked) {
    let upd = {
      checked: checked
    };
    if(checked && !this.state.toggled) {
      upd.toggled = true;
    }
    this.setState(upd);
    this.props.onChange && this.props.onChange(checked);
  }

  renderIcon() {
    if (!this.props.icon) return;
    if (typeof this.props.icon === 'object' && this.props.icon) return (
      <div className="app-checkbox-icon">{this.props.icon}</div>
    );
    return (
      <div className="app-checkbox-icon">
        <i className={this.props.icon}/>
      </div>
    )
  }

  renderContent() {
    let children = (this.props.children instanceof Array) ? this.props.children : (typeof this.props.children === 'object' && this.props.children ? [this.props.children] : null);
    if (!children) return;
    return (
      <div className="app-checkbox-group-content">
        {children}
      </div>
    );
  }

  renderMore() {
    let children = (this.props.children instanceof Array) ? this.props.children : (typeof this.props.children === 'object' && this.props.children ? [this.props.children] : null);
    if (!children) return;
    return (
      <IconButton
        onClick={() => this.setState({toggled: !this.state.toggled})}
        className="app-checkbox-more">
        {this.renderMoreIcon()}
      </IconButton>

    )
  }

  renderMoreIcon() {
    if(!this.props.toggleIcon) {
      return (
        <MoreVertIcon />
      )
    }
    if(this.state.toggled) {
      return <ExpandLessIcon />
    }
    return <ExpandMoreIcon />
  }

  render() {
    let children = (this.props.children instanceof Array) ? this.props.children : (typeof this.props.children === 'object' && this.props.children ? [this.props.children] : null),
      hasChildren = !!children;
    return (
      <div className={classNames("app-checkbox-group", {
      "group-has-icon": !!this.props.icon,
      "group-is-toggled": this.state.toggled || !hasChildren,
      'group-is-checked': this.state.checked
      },this.props.className)} style={this.props.style}>
        <div className="app-checkbox-toggle">
          <Checkbox
            name={this.props.name}
            checked={this.state.checked}
            onChange={(checked) => this.handleCheckbox(checked)}
          />
        </div>
        <div className="app-checkbox-inner">
          <div className="app-checkbox-group-top">
            {this.renderIcon()}
            <h4 onClick={this.handleTitleClick}>{this.props.title}</h4>
            {this.renderMore()}
          </div>
          {this.renderContent()}
        </div>
      </div>
    )
  }
}

CheckboxGroup.PropTypes = {
  toggled: React.PropTypes.any,
  title: React.PropTypes.any,
  icon: React.PropTypes.any,
  name: React.PropTypes.any,
  toggleIcon: React.PropTypes.any,
  onChange: React.PropTypes.func,
  checked: React.PropTypes.any
};
