'use strict';

const React = require('react'),
  moment = require('moment'),
  util = require('../../../lib/util');
import * as MuiDatePicker from 'material-ui/DatePicker';
import * as _dateUtils from 'material-ui/DatePicker/dateUtils';
import * as MuiTimePicker from 'material-ui/TimePicker';
import getMuiTheme from 'material-ui/styles/getMuiTheme';

const theme = require('theme');

function getDate(date, isTime) {
  if (!date) return '';
  if (date == null || typeof date === 'undefined') return '';
  if (date instanceof Date) date;
  if (isTime) {
    let dt = moment(),
      tmp = date.split(':');
    dt.hours(tmp[0]);
    dt.minutes(tmp[1]);
    dt.seconds(tmp[2]);
    return dt.toDate();
  }
  try {
    return moment(date).toDate();
  } catch (e) {
    return '';
  }
}

function getDateValue(date, isTime) {
  if (typeof date !== 'object' || !date) return date || '';
  if (isTime) {
    let h = date.getHours().toString(),
      m = date.getMinutes().toString(),
      s = date.getSeconds().toString();
    if (h < 10) h = '0' + h;
    if (m < 10) m = '0' + m;
    if (s < 10) s = '0' + s;
    return `${h}:${m}:${s}`;
  }
  let y = date.getFullYear(),
    m = date.getMonth() + 1,
    d = date.getDate();
  if (m < 10) m = `0${m}`;
  if (d < 10) d = `0${d}`;
  let utc = `${y}-${m}-${d}T00:00:00.000Z`;
  return utc;
}

export default class DatePicker extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      hasDate: false,
      value: getDate(props.defaultValue, props.time),
      valueString: getDateValue(props.defaultValue)
    };
    this.handleChange = this.handleChange.bind(this);
  }

  componentWillReceiveProps(props) {
    this.setState({
      value: getDate(this.state.valueString || props.defaultValue, props.time),
      valueString: getDateValue(this.state.valueString || props.defaultValue, props.time)
    });
  }

  setValue(val) {
    this.handleChange(null, val);
  }

  getValue() {
    return this.state.valueString;
  }

  handleChange(e, val) {
    let upd = {
      hasDate: !!val,
      value: val,
      valueString: getDateValue(val, this.props.time)
    };
    this.setState(upd, () => {
      this.props.onChange && this.props.onChange(upd.valueString);
      if (this.input) $(this.input).trigger('change');
    });
  }

  render() {
    let props = Object.assign({}, this.props);
    if (props.title) {
      props.floatingLabelText = props.title;
      delete props.title;
    }
    let CompType = (props.time === true) ? MuiTimePicker.default : MuiDatePicker.default;

    // TIME PICKER
    if (props.time) {
      props.dialogStyle = {
        zIndex: 2100
      };
    } else {
      // DATE PICKER
      props.dialogContainerStyle = {
        zIndex: 2100
      };
    }
    if (this.state.value) {
      props.value = this.state.value;
    }
    if (props.value) props.value = new Date(props.value);
    delete props.defaultValue;
    delete props.time;
    delete props.onChange;
    let inputName = props.name;
    if (typeof props.errorText === 'object' && props.errorText) {
      props.errorText = props.errorText[props.name];
    }
    /* Check the value is valid before doing anything */
    if (props.value) {
      let isValid = validateValue(props.value);
      if (isValid === null) {
        delete props.value;
      }
    }
    delete props.name;
    return (
      <div>
        <input ref={(c) => this.input = c} type="hidden" name={inputName} value={this.state.valueString}/>
        <CompType
          {...props}
          className={classNames("app-datepicker", {
            "has-date": this.state.hasDate,
            "is-time": this.props.time == true
          }, this.props.className)}
          fullWidth={true}
          onChange={this.handleChange}
        />
      </div>
    );
  }

  getChildContext() {
    let palette = Object.assign({}, theme.palette, {
      primary2Color: theme.palette.primary3Color,
      alternateTextColor: theme.palette.accent2Color
    });
    if (this.props.time === true) {
      palette.textColor = "#ffffff";
    }
    return {
      muiTheme: getMuiTheme({
        palette: palette,
        textField: Object.assign({}, theme.textField, {
          textColor: theme.palette.textColor
        })
      })
    };
  }
}
DatePicker.childContextTypes = {
  muiTheme: React.PropTypes.object.isRequired
};

DatePicker.PropTypes = {
  defaultValue: React.PropTypes.any,
  time: React.PropTypes.any,
  onChange: React.PropTypes.func,
  title: React.PropTypes.any,
  floatingLabelText: React.PropTypes.any,
  dialogStyle: React.PropTypes.any,
  dialogContainerStyle: React.PropTypes.any,
  name: React.PropTypes.any,
  value: React.PropTypes.any
};

function validateValue(date) {
  try {
    return (0, _dateUtils.formatIso)(date);
  } catch (e) {
    return null;
  }
}
