'use strict';
/**
 * Dialog wrapper
 */
import Dialog from 'material-ui/Dialog';
import FlatButton from 'material-ui/FlatButton';
import Button from './Button';

const React = require('react'),
  util = require('../../../lib/util');

export default class AppDialog extends React.Component {

  constructor(props) {
    super(props);
    this.handleScroll = this.handleScroll.bind(this);
    this._scrollTimer = null;
  }

  handleScroll() {
    if (this.props.disableResize !== false){
      util.triggerResize();
    }
  }

  componentDidMount() {
    $("body").addClass("app-dialog-open");
    this._scrollTimer = setTimeout(() => {
      $(".app-dialog .app-dialog-content div").on('scroll', this.handleScroll);
    }, 10);
  }

  componentWillUnmount() {
    $("body").removeClass("app-dialog-open");
    clearTimeout(this._scrollTimer);
    this._scrollTimer = null;
    $(".app-dialog .app-dialog-content div").off('scroll', this.handleScroll);
  }

  render() {
    let props = Object.assign({}, {
      scroll: true
    }, this.props);
    if (props.scroll) {
      props.autoScrollBodyContent = true;
      delete props.scroll;
    }
    let customStyle = {};
    if (props.style) {
      customStyle = Object.assign({}, customStyle, props.style);
      delete props.style;
    }
    if (props.width) {
      customStyle.width = props.width;
      customStyle.maxWidth = 'none';
      delete props.width;
    }
    if (props.minWidth) {
      customStyle.minWidth = props.minWidth;
      delete props.minWidth;
    }
    delete props.className;
    return (
      <Dialog
        modal={true}
        open={true}
        {...props}
        style={{
          zIndex: 2000,
          ...props.style
        }}
        contentStyle={customStyle}
        contentClassName="app-dialog-content"
        className={classNames("app-dialog", this.props.className)}
      />
    )
  }
}

AppDialog.Cancel = class AppDialogCancel extends React.Component {
  render() {
    return (
      <Button
        flat={true}
        label={this.props.label || "CANCEL"}
        {...this.props} />
    )
  }
};


AppDialog.Button = class AppDialogButton extends React.Component {
  render() {
    return (
      <Button
        label={this.props.label || "SAVE"}
        primary={true}
        {...this.props}
      />
    )
  }
};

AppDialog.PropTypes = {
  disableResize: React.PropTypes.bool,
  scroll: React.PropTypes.any,
  autoScrollBodyContent: React.PropTypes.any,
  width: React.PropTypes.any,
  minWidth: React.PropTypes.any
};

AppDialog.Button.PropTypes = {
  label: React.PropTypes.any
};

AppDialog.Cancel.PropTypes = {
  label: React.PropTypes.any
};
