'use strict';
/**
 * Default dropdown menu
 */
const React = require('react'),
  router = require('router');

import IconMenu from 'material-ui/IconMenu';
import MenuItem from 'material-ui/MenuItem';
import IconButton from 'material-ui/IconButton/IconButton';
import MoreVertIcon from 'material-ui/svg-icons/navigation/more-vert';

export default class Dropdown extends React.Component {

  render() {
    let icon = this.props.icon || <MoreVertIcon/>;
    // Check if we return a '-' or the icon menu
    let hasChildren = false;
    if (typeof this.props.children === 'undefined' || !this.props.children) {
      hasChildren = false;
    } else {
      if (this.props.children instanceof Array) {
        for (let i = 0; i < this.props.children.length; i++) {
          if (typeof this.props.children[i] === 'object' && this.props.children[i]) {
            hasChildren = true;
            break;
          }
        }
      } else if (typeof this.props.children === 'object' && this.props.children) {
        hasChildren = true;
      }
    }
    if (!hasChildren) {
      return <span>-</span>;
    }
    let children = [],
      rawChildren = [];
    if (this.props.children instanceof Array) {
      children = this.props.children;
    } else {
      children = [this.props.children];
    }
    for (let i = 0, len = children.length; i < len; i++) {
      let c = children[i];
      if (typeof c !== 'object' || !c) continue;
      let oldClick = c.props.onClick;
      let opt = {
        key: i,
        desktop: typeof this.props.desktop === 'undefined' ? true : this.props.desktop,
        onClick: (e) => {
          oldClick && oldClick(e);
          this.dropdown && this.dropdown.close();
        }
      };
      let itemObj = React.cloneElement(c, opt);
      rawChildren.push(itemObj);
    }
    return (
      <IconMenu
        className="app-dropdown"
        menuStyle={Object.assign({}, this.props.menuStyle || {}, {
          zIndex: 1200,
          minWidth: 128
        })}
        ref={(c) => this.dropdown = c}
        animated={true}
        useLayerForClickAway={true}
        iconButtonElement={<IconButton>{icon}</IconButton>}
        anchorOrigin={{horizontal: 'right', vertical: 'top'}}
        targetOrigin={{horizontal: 'right', vertical: 'top'}}
        {...this.props}
      >
        {rawChildren}
      </IconMenu>
    )
  }
}

Dropdown.PropTypes = {
  desktop: React.PropTypes.boolean,
  icon: React.PropTypes.any,
  menuStyle: React.PropTypes.any
};

/*
 * WORKS AS:
 * <Dropdown.Item text=Sometext onClick=fn />
 * */
Dropdown.Item = class DropdownItem extends React.Component {

  render() {
    let props = Object.assign({}, this.props);
    if (props.text) {
      props.primaryText = props.text;
      delete props.text;
    }
    if (props.disabled && props.onClick) {
      delete props.onClick;
    }
    if (props.href && !props.disabled) {
      let url = props.href;
      props.onClick = () => {
        this.props.onClick && this.props.onClick();
        router.navigate(url);
      };
      delete props.href;
    }
    return (
      <MenuItem  {...props}/>
    )
  }
};

Dropdown.Item.PropTypes = {
  text: React.PropTypes.any,
  disabled: React.PropTypes.any,
  href: React.PropTypes.any,
  onClick: React.PropTypes.func
};
