'use strict';

const React = require('react'),
  TextField = require('./TextField');

class EditableField extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      isEditing: false,
      isHover: false
    };
    this.handleCancel = this.handleCancel.bind(this);
  }

  componentDidMount() {
    $(document).on('keydown', this.handleCancel);
  }

  componentWillUnmount() {
    $(document).off('keydown', this.handleCancel);
  }

  handleCancel(e) {
    if (!this.state.isEditing) return;
    if (e.keyCode !== 27) return;
    this.setState({
      isEditing: false,
      isHover: false
    });
  }

  handleSave() {
    let field = this.textField;
    if (!field) return;
    let val = field.getValue();
    if (this.props.defaultValue && val === this.props.defaultValue) {
      return this.setState({
        isEditing: false,
        isHover: false
      });
    }
    this.props.onChange && this.props.onChange(val);
    return this.setState({
      isEditing: false,
      isHover: false
    });
  }

  renderContent() {
    let toggleStyle = {};
    if (this.toggle) {
      let $t = $(this.toggle),
        w = $t.width() + 8;
      toggleStyle.right = '-' + w + 'px';
    }
    let label = 'CHANGE';
    if (this.props.changeLabel) label = this.props.changeLabel.toUpperCase();
    return (
      <div>
        {this.props.children}
        <a
          onClick={() => this.setState({isEditing: true})}
          style={toggleStyle}
          ref={(c) => this.toggle = c}
          className={classNames("app-editable-toggle", {
            "app-editable-toggle-visible": this.state.isHover
          })}>{label}</a>
      </div>
    )
  }

  renderField() {
    return (
      <TextField
        className="app-editable-textfield small"
        name={this.props.name}
        title={this.props.title}
        delayed={true}
        defaultValue={this.props.defaultValue}
        ref={(c) => this.textField = c}
        hintText={this.props.hintText}
        errorText={this.props.errorText}
        onKeyPress={(e) => {
          if (e.charCode === 13) return this.handleSave();
        }}
        onBlur={() => this.handleSave()}
      />
    )
  }

  render() {
    return (
      <div
        className={classNames("app-editable-field", this.props.className)}
        onMouseEnter={() => this.setState({isHover: true})}
        onMouseLeave={() => this.setState({isHover: false})}
      >
        {this.state.isEditing ? this.renderField() : this.renderContent()}
      </div>
    )
  }

}

module.exports = EditableField;

EditableField.PropTypes = {
  onChange: React.PropTypes.func,
  changeLabel: React.PropTypes.string,
  title: React.PropTypes.string
};
