'use strict';
import * as Filter from 'core/components/Filter';

/**
 * Example: Filter.Search ONLY
 * */
const renderSearchOnly = (
  <Filter.Search
    className="my-class-name"
    onFilter={(filter) => {
      console.log('FILTER', filter);
    }}
    ref={(c) => this.filter = c}
  />
);

/**
 * Example: Filter.SearchFilter
 *
 * Note:
 * You can set as many Filter.Row/Filter.Cols as you want.
 * Note2: We can only display 4 cols in a row.
 * */

const renderSearchFilter = (
  <Filter.SearchFilter
    className="my-class"
    ref={(c) => this.filter = c}
    onFilter={(filter) => {
      console.log('FILTER', filter);
    }}
    labelSearch={'Do search'}
    labelClear={'Do clear'}
  >
    <Filter.Row>
      <Filter.Col>
        <Select
          ref={(c) => this.filter && this.filter.field(c)}
          name="role_id"
          defaultValue="ALL"
          title={'Select a role'}
          onChange={(v) => this.filter && this.filter.set('role_id', v)}
        >
          <Select.Item value="ALL" primaryText={_tr('generic.all')}/>
          <Select.Item value="1" primaryText={'Role 1'}/>
        </Select>
      </Filter.Col>
    </Filter.Row>
    <Filter.Row>
      <Filter.Col>
        <Select
          ref={(c) => this.filter && this.filter.field(c)}
          name="user_id"
          defaultValue="ALL"
          title={'Select a user'}
          onChange={(v) => this.filter && this.filter.set('user_id', v)}
        >
          <Select.Item value="ALL" primaryText={_tr('generic.all')}/>
          <Select.Item value="1" primaryText={'User 1'}/>
        </Select>
      </Filter.Col>
      <Filter.Col>
        <TextField
          ref={(c) => this.filter && this.filter.field(c)}
          name="image"
          defaultValue="Default image"
          title={'Enter an image'}
          onChange={(v) => this.filter && this.filter.set('image', v)}
        />
      </Filter.Col>
    </Filter.Row>
  </Filter.SearchFilter>
);


/**
 * Filter.Only EXAMPLE
 *
 * Note: it ALWAYS REQUIRES a PrimaryField
 * */
const renderFilterOnly = (
  <Filter.Only
    ref={(c) => this.filter = c}
    onFilter={(filter) => {
      console.log('FILTER', filter);
    }}
  >
    <Filter.PrimaryField
      title="Environment"
      name="environment"
      ref={(c) => this.filter && this.filter.field(c)}
      defaultValue="ALL"
    >
      <Filter.PrimaryField.Item
        value="ALL"
        primaryText="All environments"
      />
      <Filter.PrimaryField.Item
        value="development"
        primaryText="Development"
      />
      <Filter.PrimaryField.Item
        value="prod"
        primaryText="Production"
      />
    </Filter.PrimaryField>
    <Filter.Row>
      <Filter.Col>
        <Select
          ref={(c) => this.filter && this.filter.field(c)}
          name="animal_id"
          defaultValue="ALL"
          title={"Animal id"}
          onChange={(v) => this.filter && this.filter.set('animal_id', v)}
        >
          <Select.Item value="ALL" primaryText={"ALL"}/>
          <Select.Item value="armadillo" primaryText={"Armadillo"}/>
        </Select>
      </Filter.Col>
    </Filter.Row>
  </Filter.Only>
);
