'use strict';

const React = require('react'),
  BaseFilter = require('./Base'),
  FilterIcon = require('./FiltrumIcon'),
  FilterToggle = require('./FiltrumToggle'),
  FilterPrimary = require('./FiltrumPrimary'),
  FilterCol = require('./FiltrumCol'),
  Select = require('core/components/Select');

class FilterOnly extends BaseFilter {

  constructor(props) {
    super(props);
    this.type = 'filter-only';
  }

  componentDidMount() {
    this.setState({
      isFilterRendered: true
    });
  }

  clear(a, b, c) {
    if (this._primary) {
      this._primary.setValue(this._primary.props.defaultValue || null);
    }
    super.clear(a, b, c);
  }

  _renderPrimaryClone(primaryObj) {
    let items = [],
      children = primaryObj.getChildren();
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      items.push(<Select.Item
        key={i}
        primaryText={child.props.primaryText}
        secondaryText={child.props.secondaryText}
        value={child.props.value}
        disabled={child.props.disabled}
      />);
    }
    return (
      <FilterCol key="primary">
        <Select
          ref={(c) => this.field(c)}
          multiple={primaryObj.props.multiple}
          className={primaryObj.props.className}
          name={primaryObj.props.name}
          defaultValue={primaryObj.getValue()}
          title={primaryObj.props.title}
          disabled={primaryObj.props.disabled}
          onChange={(val) => {
            primaryObj.setValue(val);
            this.set(primaryObj.props.name, val);
          }}
        >
          {items}
        </Select>
      </FilterCol>
    )
  }

  /*
  * Injects the primary item in the items
  * */
  getItemsPrimary(items) {
    let result = items.concat([]),
      firstRow = items[0],
      primaryObj = this._primary;
    if (firstRow && primaryObj) {
      let rowChildren = (firstRow.props.children instanceof Array ? firstRow.props.children : [firstRow.props.children]).concat([]),
        primaryRow = this._renderPrimaryClone(primaryObj);
      rowChildren.splice(0, 0, primaryRow);
      let newProps = Object.assign({}, firstRow.props);
      newProps.key = 'cloned-row';
      newProps.children = rowChildren;
      result[0] = React.cloneElement(firstRow, newProps);
    } else {
      result = items;
    }
    return result;
  }

  getPrimaryFilter() {
    let children = (this.props.children instanceof Array ? this.props.children : [this.props.children]);
    for (let i = 0, len = children.length; i < len; i++) {
      let item = children[i];
      if (typeof item !== 'object' || !item) continue;
      if (item.type === FilterPrimary) return item;
    }
    return null;
  }

  render() {
    let items = this.getItems(),
      primary = this.getPrimaryFilter();
    if (!primary) {
      this._warn('Filter-only mode requires a Filter.PrimaryField component in the children', true);
      return (<div/>);
    }
    let isPrimaryVisible = !this.state.isFilterFocused,
      primaryObj;
    if (this.state.isFilterRendered) {
      primaryObj = React.cloneElement(primary, {
        ref: (c) => {
          this._primary = c;
          this.field(c, '__primary_field');
        },
        isVisible: isPrimaryVisible,
        onChange: (val) => {
          this.set(primary.props.name, val, () => {
            this._fields[primary.props.name] && this._fields[primary.props.name].setValue(val);
            primary.props.onChange && primary.props.onChange(val);
            if (!this.state.isFilterFocused) {
              this.filter();
            }
          });
        }
      });
    }
    items = this.getItemsPrimary(items);
    return (
      <div className={this.getClassNames(items)} style={this.props.style}>
        <div className="app-filtrum-bar">
          <div className="app-filtrum-primary-filter">
            {primaryObj}
          </div>
          {items.length > 0 && (
            <FilterToggle
              disabled={items.length === 0}
              selected={this.isToggleActive()}
              active={this.state.isFilterVisible && items.length > 0}
              onToggle={this.handleFilterVisible}
            />
          )}
          <FilterIcon
            icon={this.state.isFilterVisible ? "clear" : "filter"}
            onSearch={() => {
              if (this.state.isFilterVisible) {
                return this.clear();
              }
              return this.filter();
            }}
          />
        </div>
        {this._renderContentBlock(items)}
        {this._renderOverlay()}
      </div>
    );
  }
}


FilterOnly.defaultProps = {
  filterOpened: false,
  canClear: true,
  labelClear: 'Clear',
  labelFilter: 'Filter'
};

FilterOnly.PropTypes = {
  filterOpened: React.PropTypes.bool,
  labelClear: React.PropTypes.string,
  labelFilter: React.PropTypes.string,
  canClear: React.PropTypes.bool,
  onChange: React.PropTypes.func // the onChange(key, value) function
};

module.exports = FilterOnly;

