'use strict';
/*
* This is the Filter's PrimaryField that renders as a compact Menu.
* This will act as a Select, so we only support Selects as a primary filter
* */
const React = require('react');
const theme = require('theme');
import ExpandMore from 'material-ui/svg-icons/navigation/expand-more';
import DropDownMenu from 'material-ui/DropDownMenu';
import MenuItem from 'material-ui/MenuItem';

const STYLES = {
  root: {
    width: '100%'
  },
  underline: {
    display: 'none'
  },
  label: {
    height: 36,
    fontSize: '14px',
    lineHeight: '36px',
    paddingLeft: 10
  },
  list: {
    paddingTop: 2,
    paddingBottom: 4
  },
  menuItem: {
    marginLeft: '-14px',
    fontSize: '14px',
    minHeight: '34px'
  }
};

class FiltrumPrimary extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      value: props.defaultValue || null
    };
    this.handleChange = this.handleChange.bind(this);
  }

  getValue() {
    return this.state.value;
  }

  setValue(value) {
    this.setState({
      value
    }, () => {
      this.props.onChange && this.props.onChange(value);
    });
  }

  handleChange(e, idx, value) {
    this.setValue(value);
  }

  getChildren(value) {
    let children = (this.props.children instanceof Array ? this.props.children : [this.props.children]),
      items = [];
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (typeof child !== 'object' || !child) continue;
      if (child instanceof Array) {
        for (let j = 0, jlen = child.length; j < jlen; j++) {
          let subChild = child[j];
          if (typeof subChild === 'object' && subChild && subChild.props && subChild.props.primaryText) {
            try {
              let innerDivStyle = {};
              if (typeof value !== 'undefined') {
                if (subChild.props.value === value || (value instanceof Array && value.indexOf(subChild.props.value) !== -1)) {
                  innerDivStyle.color = theme.palette.primary1Color;
                }
              }
              subChild = React.cloneElement(subChild, {
                ...subChild.props,
                innerDivStyle
              });
            } catch (e) {
            }
            items.push(subChild);
          }
        }
        continue;
      }
      if (!child.props || !child.props.primaryText) continue;
      try {
        if (this.state.value === child.props.value) {
          let innerDivStyle = {};
          if (typeof value !== 'undefined') {
            if (child.props.value === value || (value instanceof Array && value.indexOf(child.props.value) !== -1)) {
              innerDivStyle.color = theme.palette.primary1Color;
            }
          }
          child = React.cloneElement(child, {
            ...child.props,
            innerDivStyle
          });
        }
      } catch (e) {
      }
      items.push(child);
    }
    return items;
  }

  render() {
    let _value = this.state.value;
    if (_value instanceof Array) {
      _value = _value.join(',');
    }
    let items = this.getChildren(_value),
      hasSecondaryText = false;
    for (let i = 0, len = items.length; i < len; i++) {
      let item = items[i];
      if (item.props.secondaryText) {
        hasSecondaryText = true;
        break;
      }
    }
    if (!this.props.isVisible) return <div/>;
    return (
      <div className={classNames(this.props.className)} style={this.props.style}>
        <input type="hidden" name={this.props.name} value={_value}/>
        <DropDownMenu
          ref={(c) => this._menu = c}
          className="app-filtrum-primary-menu"
          animated={this.props.animated}
          onChange={this.handleChange}
          value={this.state.value}
          autoWidth={false}
          underlineStyle={STYLES.underline}
          labelStyle={STYLES.label}
          listStyle={STYLES.list}
          iconButton={<ExpandMore/>}
          menuItemStyle={STYLES.menuItem}
          style={Object.assign({}, this.props.style || {}, STYLES.root)}
        >
          {items}
        </DropDownMenu>
      </div>
    )
  }

}

FiltrumPrimary.defaultProps = {
  animated: true,
  multiple: false
};
FiltrumPrimary.PropTypes = {
  name: React.PropTypes.string.isRequired,
  title: React.PropTypes.string.isRequired,
  isVisible: React.PropTypes.bool,
  animated: React.PropTypes.bool,
  multiple: React.PropTypes.bool,
  defaultValue: React.PropTypes.any
};

FiltrumPrimary.Item = MenuItem;


module.exports = FiltrumPrimary;
