'use strict';

const React = require('react'),
  Col = require('./FiltrumCol');

class FiltrumRow extends React.Component {

  getColumnClass(items, pos) {
    let itm = items[pos];
    if (itm.props && itm.props.fullWidth === true) {
      return 'col-xs-12';
    }
    if (items.length === 3) {
      if (pos === 0) return 'col-xs-6';
      return 'col-xs-3';
    }
    if (items.length === 4) return 'col-xs-3';
    return 'col-xs-6';
  }

  render() {
    let children = (this.props.children instanceof Array ? this.props.children : [this.props.children]),
      items = [];
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (typeof child !== 'object' || !child) continue;
      if (child.type !== Col) continue;
      items.push(child);
    }
    /* For each item, we have to clone it so that we can give them their cols */
    for (let i = 0, len = items.length; i < len; i++) {
      if (i > 4) break;
      items[i] = React.cloneElement(items[i], {
        key: i,
        className: classNames(items[i].className, this.getColumnClass(items, i))
      });
    }
    if (items.length === 0) {
      items.push(<div/>);
    }
    return (
      <div className={classNames('app-filtrum-row', {
        'app-filtrum-row-compact': this.props.compact
      }, 'row', this.props.className)}>
        {items}
      </div>
    )
  }

}

FiltrumRow.defaultProps = {
  compact: true
};

FiltrumRow.PropTypes = {
  compact: React.PropTypes.bool // If this will be set to false, we will not add the margin-top: -12px to the row.
};

module.exports = FiltrumRow;
