'use strict';
const React = require('react'),
  BaseFilter = require('./Base'),
  FilterIcon = require('./FiltrumIcon');

/**
 * The FilterSearch component is meant to be used only as a search component.
 * An example usage would be
 * import Filter from 'core/components/Fiter';
 *
 * render() {
 *  return (<Filter.Search
 *    labelSearch="Search through your documents"
 *    onFilter={(filter) => {console.log(filter.search)}}
 *  />);
 * }
 * */
class Search extends BaseFilter {

  constructor(props) {
    super(props);
    this.type = 'search';
  }

  componentDidMount() {
    if (this.props.searchValue) {
      this.handleSearch(this.props.searchValue);
      this.setState({
        hasSearchText: true
      });
    }
  }

  render() {
    return (
      <div className={this.getClassNames()} style={this.props.style}>
        <div className="app-filtrum-bar">
          {this._renderSearchField()}
          <FilterIcon onSearch={() => this.handleSearch(this._search && this._search.getValue() || '')}/>
        </div>
      </div>
    )
  }

}

Search.defaultProps = {
  searchDelayed: true,
  searchName: 'search',
  labelSearch: 'Search'
};

Search.PropTypes = {
  searchName: React.PropTypes.string,
  searchValue: React.PropTypes.string,
  searchDelayed: React.PropTypes.bool,
  searchExpand: React.PropTypes.bool, // If set to false, do not expand search on blur/focus
  labelSearch: React.PropTypes.string,
  onFilter: React.PropTypes.func.isRequired
};

module.exports = Search;
