'use strict';

const Search = require('./Search'),
  SearchFilter = require('./SearchFilter'),
  Only = require('./FilterOnly'),
  Row = require('./FiltrumRow'),
  Col = require('./FiltrumCol'),
  PrimaryField = require('./FiltrumPrimary');

/*
* The Filter component can be used in 3 states:
*   - Search Only
*   - Search & Filter
*   - Filter Only
*
* Thus, you can use it as:
* import Filter from 'core/components/Filter'
*
* <Filter.Search /> // the simple search
* <Filter.SearchFilter /> // The Search & Filter
* <Filter.Filter />       // The Filter only
* */

module.exports = {
  Search,
  SearchFilter,
  Only,
  PrimaryField,
  Row,
  Col
};

/**
 * Filter.Search EXAMPLE
 <Filter.Search
 className="bottom-24"
 onFilter={(filter) => {
      console.log('FILTER2', filter);
    }}
 ref={(c) => this.filter1 = c}
 />
 *
 * */

/**
 * Filer.SearchFilter EXAMPLE
 *
 <Filter.SearchFilter
 className="my-class-name"
 ref={(c) => this.filter = c}
 onFilter={(filter) => {
    console.log('FILTER', filter);
 }}
 labelSearch="Do filter"
 labelClear="Clear"
 >
 <Filter.Row>
 <Filter.Col>
 <Select
 ref={(c) => this.filter3 && this.filter3.field(c)}
 className="filter-select"
 name="role_id"
 defaultValue="ALL"
 title={_t('generic.role')}
 onChange={(v) => this.filter3 && this.filter3.set('role_id', v)}
 >
 <Select.Item value="ALL" primaryText={_tr('generic.all')}/>
 <Select.Item value="1" primaryText={'One'}/>
 </Select>
 </Filter.Col>
 </Filter.Row>
 </Filter.SearchFilter>
 * */
