/**
 * Created by zsoltmakai on 9/14/2016.
 */
'use strict';
const React = require('react'),
  tredux = require('tredux'),
  util = require('lib/util'),
  router = require('router');
export default class Form extends React.Component {

  constructor(props) {
    super(props);
    this.submit = this.submit.bind(this);
    this.handleFormChange = this.handleFormChange.bind(this);
    this.submit = this.submit.bind(this);
  }

  componentDidMount() {
    let $form = $(this.refs.form);
    $form.on('change', ':input, select, textarea', this.handleFormChange);
  }

  componentWillUnmount() {
    let $form = $(this.refs.form);
    $form.off('change', ':input, select, textarea', this.handleFormChange);
  }

  handleFormChange(e) {
    let $target = $(e.target);
    if (!$target.attr('name')) return;
    this.props.onChange && this.props.onChange(this.getPayload());
  }


  getPayload() {
    let $form = $(this.refs.form),
      payload = {},
      data = $form.serializeArray();
    // check if we have input type checkbox
    $form.find("input[type='checkbox']").each((id, item) => {
      let name = $(item).attr("name"),
        value = $(item).val();
      if (!name) return;
      if (value === "on" || value === "off" || typeof value !== 'string') {
        payload[name] = item.checked;
        return;
      }
      if (item.checked) {
        if (!payload[name]) payload[name] = [];
        payload[name].push(value);
      }
    });
    data.forEach((item) => {
      if (typeof payload[item.name] !== 'undefined') return;
      payload[item.name] = item.value;
    });
    $form.find('input[data-type="array"]').each((id, itm) => {
      let value = $(itm).val(),
        name = $(itm).attr('name');
      if (!name) return;
      if (!value) {
        payload[name] = [];
        return;
      }
      value = value.split(',');
      payload[name] = value;
    });
    return payload;
  }

  submit(e) {
    if (e) {
      e.preventDefault();
      e.stopPropagation();
    }
    this.props.onSubmit && this.props.onSubmit(this.getPayload());
    return false;
  }


  render() {
    return (
      <form
        className={classNames("app-form", this.props.className)}
        ref="form"
        name={this.props.name || "form"}
        style={this.props.style}
        onSubmit={this.submit}>
        {this.props.children}
      </form>
    )
  }
}

Form.PropTypes = {
  onChange: React.PropTypes.func,
  onSubmit: React.PropTypes.func,
  name: React.PropTypes.any
};


/*
* Form utility class
* */
Form.Base = class FormBase extends React.Component {

  constructor(props) {
    super(props);
    this.hasErrors = false;
    if (!this.state) this.state = {};
    this._fieldErrors = {};
    this.state.errors = {};
    this.fieldError = this.fieldError.bind(this);
    this.clearErrors = this.clearErrors.bind(this);
    this.handleEscape = this.handleEscape.bind(this);
  }

  handleEscape(e) {
    if (!e || e.keyCode === 27) {
      return this.handleClose();
    }
  }

  componentDidMount() {
    if (typeof this.handleClose === 'function') {
      document.addEventListener("keydown", this.handleEscape, false);
    }
  }

  componentWillUnmount() {
    clearTimeout(this._fieldTimer);
    if (typeof this.handleClose === 'function') {
      document.removeEventListener("keydown", this.handleEscape, false);
    }
  }

  clearErrors() {
    this.hasErrors = false;
    this._fieldErrors = {};
    clearTimeout(this._fieldTimer);
    this._fieldTimer = setTimeout(() => {
      this.setState({
        errors: this._fieldErrors
      });
      this._fieldErrors = {};
    }, 1);
  }

  /* Sets a field error */
  fieldError(key, msg) {
    // Check if error
    if (typeof key === 'object' && key) {
      if (typeof key.data === 'object' && key.data && typeof key.data.field === 'string' && typeof key.message === 'string') {
        msg = key.message;
        key = key.data.field;
      } else {
        return util.error(key.message);
      }
    } else if (typeof key === 'string' && typeof msg === 'undefined') {
      if (typeof this._fieldErrors === 'object' && this._fieldErrors) {
        delete this._fieldErrors[key];
      }
    } else if (typeof key === 'string' && typeof msg === 'string') {
      this._fieldErrors[key] = msg;
    }
    let oldErrors = {};
    if (this._fieldTimer) {
      clearTimeout(this._fieldTimer);
      oldErrors = this._fieldErrors;
    }
    this._fieldErrors = Object.assign({}, oldErrors);
    if (typeof msg !== 'undefined') {
      this._fieldErrors[key] = msg;
    }
    if (Object.keys(this._fieldErrors).length === 0) {
      this.hasErrors = false;
    } else {
      this.hasErrors = true;
    }
    this._fieldTimer = setTimeout(() => {
      this.setState({
        errors: this._fieldErrors
      }, () => {
        this._fieldErrors = this.state.errors;
      });
    }, 5);
  }
};
