/**
 * Created by zsoltmakai on 9/14/2016.
 */
'use strict';
const React = require('react'),
  PropTypes = React.PropTypes,
  tredux = require('tredux');

import TextField from 'material-ui/TextField';
import Button from './Button';
import Form from './Form';

export default class FormFilter extends React.Component {

  constructor(props) {
    super(props);
    console.warn(`Tredux-material.FormFilter: deprecated. Please use 'core/components/Filter'`)
    this.handleSearch = this.handleSearch.bind(this);
    this.handleForm = this.handleForm.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.state = {
      isToggled: (typeof props.opened === 'boolean' ? props.opened : false),
      isFocused: false
    }
  }

  componentWillReceiveProps(props) {
    if (typeof props.opened === 'boolean') {
      this.setState({
        isToggled: props.opened
      });
    }
  }

  componentWillUnmount() {
    if (this._timer) {
      clearTimeout(this._timer);
      this._timer = null;
    }
  }

  getPayload(obj) {
    let form = this.refs.form;
    let payload = form.getPayload();
    if (!payload) payload = {};
    if (typeof obj === 'object' && obj) {
      payload = Object.assign({}, obj, payload);
    }
    Object.keys(payload).forEach((keyName) => {
      if (payload[keyName] === '' || payload[keyName] === 'ALL') {
        delete payload[keyName];
      }
    });
    return payload;
  }

  handleReset() {
    this.props.onFilter && this.props.onFilter({});
  }

  handleForm(payload) {
    if (this._timer) {
      clearTimeout(this._timer);
      this._timer = null;
    }
    this.props.onFilter && this.props.onFilter(this.getPayload(payload));
  }

  handleChange() {
    this.props.onChange && this.props.onChange(this.getPayload());
  }

  handleSearch(e, term) {
    if (this._timer) {
      clearTimeout(this._timer);
      this._timer = null;
    }
    if (typeof term === 'undefined') { // direct click.
      return this.handleForm();
    }
    this._timer = setTimeout(() => {
      this.handleForm();
      this._timer = null;
    }, 400);
  }

  hasContent() {
    if (typeof this.props.children === 'undefined') return false;
    if (this.props.children instanceof Array) {
      for (let i = 0; i < this.props.children.length; i++) {
        if (typeof this.props.children[i] === 'object') return true;
      }
    }
    if (typeof this.props.children === 'object' && this.props.children) return true;
    return false;
  }

  renderSearch() {
    let searchView;
    if (this.props.search) {
      searchView = (
        <div className="form-search-field">
          {typeof this.props.search === 'function' ? this.props.search() : this.props.search}
        </div>
      );
    } else {
      searchView = (<TextField
        ref="textField"
        className={classNames("app-filter-search", {
          'is-focused': this.state.isFocused
        })}
        name="search"
        type="text"
        underlineShow={false}
        fullWidth={true}
        hintText={this.props.searchText}
        onChange={this.handleSearch}
        onFocus={() => this.setState({isFocused: true})}
        onBlur={() => this.setState({isFocused: false})}
      />);
    }

    return (
      <div className="form-filter-search" style={this.props.children ? (this.state.isToggled ? {
        width: 'calc(100% - 229px)',
        maxWidth: '792px'
      } : {width: 'calc(100% - 221px)', maxWidth: '792px'}) : {width: '100%'}}>
        {searchView}
      </div>
    )
  }


  renderSearchButton() {
    if (this.state.isToggled) return;
    return (
      <Button
        label={this.props.searchLabel}
        onClick={this.handleSearch}
        style={this.props.children ? {margin: '0 24px'} : {margin: '0 8px 0 24px'}}
      />
    )
  }

  renderContent() {
    if (!this.hasContent()) return;
    return (
      <div className={classNames("app-filter-content", {
        'app-filter-hidden': !this.state.isToggled
      })}>
        {this.props.children}
        <Button.Group align="right" className="col-xs-12 top-8">
          <Button label={this.props.resetLabel} flat={true} onClick={() => this.handleReset()}/>
          <Button label={this.props.filterLabel} onClick={() => this.handleForm()}/>
        </Button.Group>
      </div>
    )
  }

  render() {
    return (
      <div className={classNames("app-filter", this.props.className)}>
        <Form ref="form" onSubmit={this.handleForm} onChange={this.handleChange}>
          <div>
            <div className="no-content">
              {this.renderSearch()}
              {this.renderSearchButton()}
              {this.hasContent() ? (

                <a
                  className="app-filter-toggle"
                  onClick={(e) => {
                    e.preventDefault();
                    e.stopPropagation();
                    this.setState({
                      isToggled: !this.state.isToggled
                    });
                  }}
                  style={{minWidth: '85px', textAlign: 'right', display: 'block'}}
                >
                  {this.state.isToggled ? this.props.hideFilterLabel : this.props.showFilterLabel}
                </a>

              ) : ""}
            </div>
            {this.renderContent()}
          </div>
        </Form>
      </div>
    );
  }
}

FormFilter.defaultProps = {
  searchLabel: 'Search',
  filterLabel: 'Filter',
  hideFilterLabel: 'Hide filter',
  showFilterLabel: 'Show filter',
  resetLabel: 'Reset'
};

FormFilter.PropTypes = {
  searchText: PropTypes.string,
  searchLabel: PropTypes.string,
  filterLabel: PropTypes.string,
  hideFilterLabel: PropTypes.string,
  showFilterLabel: PropTypes.string,
  resetLabel: PropTypes.string,
  onFilter: PropTypes.func,
  opened: PropTypes.any,
  onChange: PropTypes.func
};
