'use strict';
/**
 * Select wrapper
 */
const React = require('react'),
  theme = require('theme'),
  util = require('../../../lib/util');
import SelectField from 'material-ui/SelectField';

const UMenuItem = require('material-ui/MenuItem').default;

export default class Select extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      selected: (typeof props.value === 'undefined' ? null : props.value)
    };
    if (typeof props.defaultValue !== 'undefined' && typeof props.value === 'undefined') {
      this.state.selected = props.defaultValue;
    }
    this.handleChange = this.handleChange.bind(this);
    this._isControlled = Object.keys(props).indexOf('value') !== -1;
  }

  getValue() {
    return this.state.selected;
  }

  setValue(val) {
    this.setState({
      selected: val
    }, () => {
      $(this.refs.input).trigger('change');
    });
  }

  handleChange(e, idx, val) {
    this.setState({
      selected: val
    }, () => {
      this.props.onChange && this.props.onChange(val);
      $(this.refs.input).trigger('change');
      util.triggerResize();
    });
  }

  componentWillReceiveProps(props) {
    if (this._isControlled && props.value !== this.state.selected) {
      this.setState({
        selected: typeof props.value === 'undefined' ? null : props.value
      }, () => {
        util.triggerResize();
      });
    }
  }

  render() {
    let props = Object.assign({}, {
      floatingLabelText: this.props.title,
      fullWidth: true,
      iconStyle: {
        fill: '#d9d9d9'
      }
    }, this.props);
    props.selectedMenuItemStyle = {
      color: theme.palette.primary1Color
    };
    if(this.props.selectedMenuItemStyle) {
      props.selectedMenuItemStyle = this.props.selectedMenuItemStyle;
    }
    delete props.title;
    if (typeof props.errorText === 'object' && props.errorText && props.name) {
      props.errorText = props.errorText[props.name] || '';
    }
    if (this.state.selected !== null) {
      props.value = this.state.selected;
    }
    let rootClass, rootStyle;
    if (props.containerStyle) {
      rootStyle = props.containerStyle;
      delete props.containerStyle;
    }
    if (props.containerClassName) {
      rootClass = props.containerClassName;
      delete props.containerClassName;
    }
    let inputProps = {
      ref: 'input',
      type: 'hidden',
      name: this.props.name,
      value: this.state.selected instanceof Array ? this.state.selected.join(',') : this.state.selected || ''
    };
    if (this.props.multiple) {
      inputProps['data-type'] = 'array';
    }
    return (
      <div className={rootClass} style={rootStyle}>
        <input {...inputProps} />
        <SelectField
          {...props}
          onChange={this.handleChange}
          className={classNames('app-select', props.className)}
        >
          {this.props.children}
        </SelectField>
      </div>
    );
  }
}

Select.PropTypes = {
  name: React.PropTypes.string,
  value: React.PropTypes.any,
  defaultValue: React.PropTypes.any,
  onChange: React.PropTypes.func,
  title: React.PropTypes.any,
  errorText: React.PropTypes.any,
  containerClassName: React.PropTypes.any,
  containerStyle: React.PropTypes.any
};

Select.Item = class MenuItem extends UMenuItem {

  constructor(props) {
    super(props);
    this.refs = {};
  }

  componentDidMount() {
    this.props.onLoad && this.props.onLoad(this);
  }

  componentWilLUnmount() {
    this.props.onLoad && this.props.onLoad(null);
  }

  render() {
    let props = Object.assign({}, this.props),
      subClasses = [];
    if (props.title) {
      props.primaryText = props.title;
      delete props.title;
    }
    if (Object.keys(props).indexOf('isChecked') !== -1) {
      subClasses.push('app-select-checker');
      props.insetChildren = true;
      props.checked = props.isChecked;
      delete props['isChecked'];
    }
    if (typeof this.state === 'object' && this.state && typeof this.state.style === 'object' && this.state.style) {
      if (typeof props.style === 'object' && props.style) {
        props.style = Object.assign({}, props.style, this.state.style);
      } else {
        props.style = this.state.style;
      }
    }
    return (
      <UMenuItem
        {...props}
        ref={(c) => this.refs = c && c.refs || {}}
        className={classNames('app-select-item', this.props.className, subClasses)}
      />
    );
  }
};

Select.Item.PropTypes = {
  title: React.PropTypes.any,
  primaryText: React.PropTypes.any
};
