'use strict';

const React = require('react');
import * as MuiSlider from 'material-ui/Slider';

export default class Slider extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      value: parseInt(props.value || props.defaultValue || 0)
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleStop = this.handleStop.bind(this);
  }

  handleChange(e, value) {
    this.setState({
      value: parseInt(value)
    });
  }

  handleStop() {
    this.props.onChange && this.props.onChange(this.state.value);
  }

  componentWillReceiveProps(props) {
    if (typeof props.value !== 'undefined' && props.value != this.state.value) {
      this.setState({
        value: parseInt(props.value)
      });
    }
  }

  render() {
    let props = Object.assign({}, this.props);
    props.sliderStyle = Object.assign({}, {
      marginBottom: 24
    }, this.props.sliderStyle || {});
    delete props.onChange;
    if (props.label) delete props.label;
    if (typeof props.defaultValue !== 'undefined') props.defaultValue = parseInt(props.defaultValue);
    if (typeof props.value !== 'undefined') props.value = parseInt(props.value);
    return (
      <div className="app-slider">
        <MuiSlider.default
          {...props}
          className="app-slider-item"
          step={this.props.step || 1}
          onDragStop={this.handleStop}
          onChange={this.handleChange}
        />
        <div className="app-slider-value">
          <span>{this.state.value}</span> {this.props.label}
        </div>
      </div>

    )
  }
};
Slider.PropTypes = {
  value: React.PropTypes.any,
  defaultValue: React.PropTypes.any,
  onChange: React.PropTypes.func,
  sliderStyle: React.PropTypes.any,
  label: React.PropTypes.any,
  step: React.PropTypes.any
};
