'use strict';

const React = require('react');

import Button from 'core/components/Button';
import Paper from 'material-ui/Paper';

class Summary extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      isActive: false,
      hasCancel: true
    };
    this.show = this.show.bind(this);
    this.hide = this.hide.bind(this);
    this.confirm = this.confirm.bind(this);
    this.cancel = this.cancel.bind(this);
    this._showFn = null;
  }

  componentDidMount() {
    this.props.onLoad && this.props.onLoad(this);
  }

  componentWillUnmount() {
    this.props.onLoad && this.props.onLoad(null);
    if (typeof this._showFn === 'function') {
      this._showFn(false);
      this._showFn = null;
    }
  }

  confirm() {
    this.props.onConfirm && this.props.onConfirm();
    this.hide(true);
  }

  cancel() {
    this.props.onCancel && this.props.onCancel();
    this.hide(false);
  }

  show(_showFn) {
    let upd = {
      isActive: true
    };
    if (typeof _showFn !== 'function') {
      upd.hasCancel = false;
    }
    this.setState(upd, () => {
      if (typeof _showFn === 'function') {
        this._showFn = _showFn;
      }
      this.props.onShow && this.props.onShow();
    });
  }

  hide(active) {
    this.setState({
      isActive: false,
      hasCancel: true
    }, () => {
      if (typeof this._showFn === 'function') {
        this._showFn(active);
        this._showFn = null;
      }
      this.props.onHide && this.props.onHide();
    });
  }

  render() {
    if (!this.state.isActive) return (<div/>);
    let _children = (this.props.onRender ? this.props.onRender() : this.props.children);
    return (
      <Paper className="app-stepper-summary">
        <h5>
          {this.props.title}
        </h5>
        <div className="app-stepper-summary-content">
          {_children}
        </div>
        <div className="app-stepper-summary-actions">
          {this.state.hasCancel && (
            <Button
              flat={true}
              label={this.props.labelCancel}
              onClick={this.cancel}
            />
          )}
          <Button
            className="app-stepper-action-confirm"
            flat={this.props.confirmFlat}
            label={this.props.labelConfirm}
            onClick={this.confirm}
          />
        </div>
      </Paper>
    );
  }

}

Summary.PropTypes = {
  title: React.PropTypes.string,
  labelConfirm: React.PropTypes.string,
  labelCancel: React.PropTypes.string,
  confirmFlat: React.PropTypes.bool,
  onRender: React.PropTypes.func
};
Summary.defaultProps = {
  title: 'Summary',
  labelConfirm: 'Confirm',
  labelCancel: 'Cancel',
  confirmFlat: true
};
module.exports = Summary;
