'use strict';
/**
 * Wrapper with default table functionality
 */
import {
  Table as MuiTable,
  TableBody as MuiTableBody,
  TableFooter as MuiTableFooter,
  TableHeader as MuiTableHeader,
  TableHeaderColumn as MuiTableHeaderColumn,
  TableRow as MuiTableRow,
  TableRowColumn as MuiTableRowColumn
} from 'material-ui/Table';
import IconButton from 'material-ui/IconButton';
import FontIcon from 'material-ui/FontIcon';

const React = require('react'),
  router = require('router');
let idx = 0;

function getTextWidth(text) {
  let pa = document.body;
  let who = document.createElement('div');
  who.style.cssText = 'display:inline-block; padding:0; line-height:1; position:absolute; visibility:hidden; font-size:16px';
  who.appendChild(document.createTextNode(text));
  pa.appendChild(who);
  let width = who.offsetWidth;
  who.remove();
  return width;
}

function getChildText($p, text = []) {
  let children = $p.children();
  if (children.length === 0) {
    return $p.text();
  } else {
    children.each(function () {
      let $c = $(this);
      text.push($c.text());
      getChildText($c, text);
    });
  }
  return text.join(' ');
}

export default class Table extends MuiTable {

  componentDidMount() {
    try {
      this.adjustResize();
    } catch (e) {
    }
  }

  componentWillReceiveProps() {
    this._timer = setTimeout(() => {
      clearTimeout(this._timer);
      try {
        this.adjustResize();
      } catch (e) {
      }
    }, 1);
  }

  adjustResize() {
    if (!this.props.adjustSize) return;
    if (typeof window.$ !== 'function' || !this.root) return;
    const MAX_SIZE_INCREMENT = 2.2; // max 2.2 times the default width.
    let $t = window.$(this.root),
      $header = $t.find('thead'),
      $body = $t.find('tbody');
    let theaders = $header.find('th');
    if (theaders.length <= 2) return;
    let rows = $body.find('tr');
    if (rows.length === 0 || rows.find('.table-no-rows').length !== 0) return;
    let columnSizeMap = {},
      columnDefaultSize = {},
      columnElementMap = {};
    theaders.each(function (idx) {
      let $td = $(this);
      let originalSize = $td.attr('data-original-width');
      if (!originalSize) {
        originalSize = $td.width();
        $td.attr('data-original-width', $td.width());
      }
      originalSize = parseInt(originalSize);
      columnDefaultSize[idx] = columnSizeMap[idx] = originalSize;
      if (!columnElementMap[idx]) columnElementMap[idx] = [];
      columnElementMap[idx].push($td);
    });
    rows.each(function () {
      let $r = $(this);
      let cols = $r.find('td');
      cols.each(function (idx) {
        let $col = $(this);
        if (!columnElementMap[idx]) columnElementMap[idx] = [];
        columnElementMap[idx].push($col);
        let text = getChildText($col),
          width = getTextWidth(text);
        if (!width) return;
        let pLeft = parseInt(($col.css('padding-left') || '0px').replace('px', ''), 10),
          pRight = parseInt(($col.css('padding-right') || '0px').replace('px', ''), 10);
        width += pLeft + pRight;
        let max = Math.min(width, columnDefaultSize[idx] * MAX_SIZE_INCREMENT);
        columnSizeMap[idx] = Math.max(columnSizeMap[idx], max);
      });
    });
    Object.keys(columnElementMap).forEach((poz) => {
      let items = columnElementMap[poz],
        width = columnSizeMap[poz];
      if (!width) return;
      for (let i = 0; i < items.length; i++) {
        let $item = items[i];
        $item.width(width);
      }
    });
  }

  render() {
    let hasPagination = false,
      hasNoRows = false;
    if (this.props.children instanceof Array) {
      for (let i = 0; i < this.props.children.length; i++) {
        let cObj = this.props.children[i];
        if (typeof cObj !== 'object' || !cObj) continue;
        if (cObj.type === Table.Pagination) {
          if (cObj.props.children || typeof cObj.props.nextPage === 'boolean') {
            hasPagination = true;
          } else if (cObj.props.items instanceof Array && cObj.props.items > cObj.props.pageSize) {
            hasPagination = true;
          } else if (cObj.props.meta && cObj.props.meta.page_count > 1) {
            hasPagination = true;
          } else if (cObj.props.meta && cObj.props.meta.page_count === 1 && cObj.props.meta.current_page > 1) {
            hasPagination = true;
          }
        }
        if (!cObj.props || !cObj.props.children) continue;

        if (cObj.type === Table.Body) {
          let bodyChildren = [];
          if (cObj.props.children instanceof Array) {
            for (let j = 0; j < cObj.props.children.length; j++) {
              let t = cObj.props.children[j];
              if (t instanceof Array) {
                bodyChildren = bodyChildren.concat(t);
              } else if (typeof t === 'object' && t) {
                bodyChildren.push(t);
              }
            }
          } else if (cObj.props.children) {
            bodyChildren.push(cObj.props.children);
          }
          for (let j = 0; j < bodyChildren.length; j++) {
            let tc = bodyChildren[j];
            if (!tc) continue;
            if (tc.type === Table.NoRows) {
              hasNoRows = true;
            }
          }
        }
      }
    }
    return (
      <div
        ref={(c) => this.root = c}
        className={classNames("app-table", {
          "app-table-no-pagination": !hasPagination,
          "app-table-has-pagination": hasPagination,
          "app-table-no-rows": !hasNoRows
        })}>
        <MuiTable {...this.props}>{this.props.children}</MuiTable>
      </div>
    )
  }
}

Table.defaultProps = {
  selectable: false,
  adjustSize: true
};
Table.PropTypes = {
  selectable: React.PropTypes.any,
  adjustSize: React.PropTypes.bool
};

/**
 * This will be our wrapped table for tables with actions that are positioned-absolute on the right
 * */
class TableActions extends React.Component {

  constructor(props) {
    super(props);
    this.itemsPerPage = 10;
    this.state = {
      actionStart: this.setPositions(props.filter, true)
    };
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.filter) {
      this.setPositions(nextProps.filter);
    }
  }

  setPositions(filter, isReturn) {
    if (!filter) return 0;
    let start = ((filter.page || 1) - 1) * this.itemsPerPage;
    if (isReturn) return start;
    this.setState({
      actionStart: start
    });
  }

  render() {
    let actions = this.props.actions || [],
      hasNoRows = true,
      actionIdxPrefix = '';
    let childTable = this.props.children;
    if (childTable && childTable.type === TablePaginated) {
      this.itemsPerPage = childTable.props.pageSize || 10;
      childTable = React.cloneElement(childTable, {
        ...childTable.props,
        onFilterChange: (filter) => this.setPositions(filter)
      });
      let start = this.state.actionStart || 0;
      actions = actions.concat([]); // clone the actions
      actions = actions.splice(start, this.itemsPerPage);
      actionIdxPrefix = start + '' + this.itemsPerPage;
      hasNoRows = false;
    }
    return (
      <div className={classNames("app-table-actions", {
        "app-table-actions-empty": actions.length === 0
      })}>
        <div className={'app-table-actions-left'}>
          {childTable}
        </div>
        {actions.length > 0 && (
          <div className={'app-table-actions-right'}>
            <Table className={classNames("striped", this.props.tableClassName)}>
              <Table.Header>
                <Table.Row>
                  <Table.HeaderCol width={8}>{this.props.header}</Table.HeaderCol>
                </Table.Row>
              </Table.Header>
              <Table.Body>
                {hasNoRows && (
                  <Table.NoRows if={false}></Table.NoRows>
                )}
                {actions.map((item, idx) => (
                  <Table.Row key={actionIdxPrefix + idx}>
                    <Table.Col width={8} className="align-center app-table-actions-col">
                      {item}
                    </Table.Col>
                  </Table.Row>
                ))}
              </Table.Body>
            </Table>
          </div>
        )}
      </div>
    )
  }
}

TableActions.PropTypes = {
  header: React.PropTypes.string,
  actions: React.PropTypes.array,
  tableClassName: React.PropTypes.any
};

Table.Actions = TableActions;

/**
 * This is our client-side paginated table.
 * It will return a wrapper over the original table,
 * making sure we have the correct meta{} and items
 * */
class TablePaginated extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      items: props.items,
      filter: typeof props.filter === 'object' && props.filter ? props.filter : {}
    };
    this.state.meta = this.getMeta(this.state.filter, props.items);
    this.getMeta = this.getMeta.bind(this);
    this.getPosition = this.getPosition.bind(this);
  }

  computerDidMount() {
    if (typeof this.props.filter === 'function') {
      let f = this.props.filter();
      if (typeof f === 'object' && f) {
        this.state.filter = f;
      }
    }
  }

  componentWillReceiveProps(nextProps) {
    let upd = {
      items: nextProps.items
    };
    if (nextProps.filter) {
      if (typeof nextProps.filter === 'function') {
        upd.filter = nextProps.filter();
      } else if (typeof nextProps.filter === 'object' && nextProps.filter) {
        upd.filter = nextProps.filter;
      }
    }
    if (this.props.resetOnUpdate === true) {
      upd.meta = this.getMeta({}, nextProps.items);
      upd.filter = {};
    }
    let hasChanges = false;
    Object.keys(upd).forEach((k) => {
      let val = JSON.stringify(upd[k]),
        current = JSON.stringify(this.state[k] || {});
      if (val !== current) {
        hasChanges = true;
      }
    });
    if (this._ignoreChanges === true) {
      delete this._ignoreChanges;
      return;
    }
    if (!hasChanges) return;
    this.setState(upd, () => {
      this.props.onFilterChange && this.props.onFilterChange(upd.filter);
    });
  }

  componentWillUnmount() {
    this._destroyed = true;
    clearTimeout(this._timer);
  }

  reset() {
    clearTimeout(this._timer);
    this._timer = setTimeout(() => {
      if (this._destroyed) return;
      this.setState({
        meta: this.getMeta({}),
        filter: {}
      }, () => {
        this.props.onFilterChange && this.props.onFilterChange({});
      });
    }, 50);
  }


  getMeta(filter, items) {
    if (!filter) filter = this.state.filter;
    if (!items) items = this.state.items;
    let meta = {
      page_count: Math.ceil(items.length / this.props.pageSize),
      total_count: items.length,
      current_count: 0,
      current_page: filter.page || 1
    }
    return meta;
  }

  /*
  * Returns the  {start, end} position for the table
  * */
  getPosition() {
    const {pageSize} = this.props;
    const {items, meta} = this.state;
    if (items.length === 0 || items.length < pageSize) {
      return {
        start: 0,
        end: items.length
      }
    }
    let start = 0, end = 0;
    start = (meta.current_page - 1) * pageSize;
    end = start + pageSize;
    return {
      start,
      end
    };
  }

  renderBody() {
    const {items, meta, filter} = this.state;
    if (items.length === 0) {
      return (
        <Table.Body>
          <Table.NoRows if={items}>{this.props.noRows}</Table.NoRows>
        </Table.Body>
      )
    }
    let rows = [],
      pos = this.getPosition();
    if (pos) {
      for (let i = pos.start; i < pos.end; i++) {
        let row = items[i];
        if (typeof row === 'undefined') break;
        idx++;
        let rowView = this.props.onRow(row, idx, i);
        if (rowView) rows.push(rowView);
      }
    }
    return (
      <Table.Body>
        {rows}
      </Table.Body>
    )
  }

  renderPagination() {
    let meta = this.getMeta();
    return (
      <Table.Pagination
        meta={meta}
        onFilter={(filter) => {
          this._ignoreChanges = true;
          this.setState({
            filter,
            meta: this.getMeta(filter)
          }, () => {
            this.props.onFilterChange && this.props.onFilterChange(filter);
            this.props.onFilter && this.props.onFilter(filter);
          });
        }}
      />
    );
  }

  render() {
    return (
      <Table className={this.props.className}>
        {this.props.header}
        {this.renderBody()}
        {this.renderPagination()}
      </Table>
    )
  }

}

TablePaginated.defaultProps = {
  pageSize: 10,
  resetOnUpdate: true
};

TablePaginated.PropTypes = {
  resetOnUpdate: React.PropTypes.bool,
  pageSize: React.PropTypes.number,
  onFilter: React.PropTypes.func,
  filter: React.PropTypes.object,
  items: React.PropTypes.array.isRequired,
  noRows: React.PropTypes.string,
  onRow: React.PropTypes.func.isRequired,
  header: React.PropTypes.any.isRequired
};

Table.Paginated = TablePaginated;

Table.Body = class TableBody extends MuiTableBody {

  render() {
    return (
      <MuiTableBody{...this.props}>{this.props.children}</MuiTableBody>
    )
  }
};

Table.Body.defaultProps = {
  displayRowCheckbox: false,
  deselectOnClickaway: false,
  showRowHover: true,
  stripedRows: false
};

Table.Body.PropTypes = {
  displayRowCheckbox: React.PropTypes.any,
  deselectOnClickaway: React.PropTypes.any,
  showRowHover: React.PropTypes.any,
  stripedRows: React.PropTypes.any
};

Table.Footer = class TableFooter extends MuiTableFooter {

  render() {
    return (
      <MuiTableFooter {...this.props}>{this.props.children}</MuiTableFooter>
    );
  }
};

Table.Header = class TableHeader extends MuiTableHeader {

  render() {
    return (
      <MuiTableHeader{...this.props}>{this.props.children}</MuiTableHeader>
    )
  }
};
Table.Header.defaultProps = {
  displaySelectAll: false,
  adjustForCheckbox: false,
  enableSelectAll: false
};

Table.Header.PropTypes = {
  displaySelectAll: React.PropTypes.any,
  adjustForCheckbox: React.PropTypes.any,
  enableSelectAll: React.PropTypes.any
};

Table.HeaderCol = class TableHeaderColumn extends MuiTableHeaderColumn {

  render() {
    let props = Object.assign({}, this.props),
      isLarge = !!props.large,
      style = buildStyles(props);
    delete props.large;
    return (
      <MuiTableHeaderColumn {...props} style={style}>
        {isLarge ?
          <h2 className={this.props.className}>
            {this.props.children}
          </h2> :
          <h3 className={this.props.className}>
            {this.props.children}
          </h3>
        }

      </MuiTableHeaderColumn>
    )
  }
};

Table.HeaderCol.PropTypes = {
  large: React.PropTypes.any
};

Table.Row = class TableRow extends MuiTableRow {

  render() {
    const props = Object.assign({}, this.props),
      styles = buildStyles(props);
    let hasClick = false;
    if (props.href) {
      let url = props.href;
      props.onClick = () => router.navigate(url);
      delete props.href;
    }
    if (props.onClick) {
      let fn = props.onClick;
      props.onMouseUp = (e) => {
        let $target = $(e.target);
        let $box = $target.closest('.app-dropdown');
        if ($box.length !== 0) return;
        let $href = $target.closest('a[href]');
        if ($href.length !== 0) return;
        e.preventDefault();
        e.stopPropagation();
        if (e && e.nativeEvent && e.nativeEvent.which == 1) {
          fn(e);
        }
        return false;
      };
      delete props.onClick;
      hasClick = true;
    }
    return (
      <MuiTableRow {...props} style={styles} className={classNames('table-row-height', {
        "table-row-hover": hasClick
      }, this.props.className)}>{this.props.children}</MuiTableRow>
    )
  }
};

Table.Row.PropTypes = {
  onClick: React.PropTypes.func,
  onMouseUp: React.PropTypes.func,
};

Table.Col = class TableRowColumn extends MuiTableRowColumn {

  render() {
    const props = Object.assign({}, this.props),
      styles = buildStyles(props);
    return (
      <MuiTableRowColumn {...props} style={styles}>{this.props.children}</MuiTableRowColumn>
    )
  }
};


Table.NoRows = class TableNoRows extends MuiTableRow {
  constructor(props) {
    super(props);
    this.state.left = null;
    this.setScrollLeft = this.setScrollLeft.bind(this);
  }

  setScrollLeft() {
    let val = this.getLeft();
    if (!val) return;
    if (val === this.state.left) return;
    this.setState({
      left: val
    });
  }

  getLeft() {
    if (!this.row) return null;
    let $row = $(this.row),
      $parent = $row.closest('.app-table');
    if (!$parent || $parent.length === 0) return null;
    let parentWidth = $parent.width(),
      scrollLeft = $parent.scrollLeft() || 0,
      textWidth = $row.width(),
      left = (parentWidth - textWidth) / 2 + scrollLeft;
    left = parseInt(left);
    return left;
  }

  componentDidMount() {
    if (this.row) {
      let $parent = $(this.row).closest('.app-table');
      $parent.on('scroll', this.setScrollLeft);
    }
    $(window).on('resize', this.setScrollLeft);
    this.setScrollLeft();
  }

  componentWillReceiveProps() {
    setTimeout(() => {
      this.setScrollLeft();
    }, 50);
  }

  componentWillUnmount() {
    $(window).off('resize', this.setScrollLeft);
    if (this.row) {
      let $parent = $(this.row).closest('.app-table');
      $parent.off('scroll', this.setScrollLeft);
    }
  }

  render() {
    const ifData = this.props.if;
    let isVisible = false;
    if (ifData instanceof Array && ifData.length === 0) {
      isVisible = true;
    } else if (typeof ifData === 'object' && ifData && Object.keys(ifData).length === 0) {
      isVisible = true;
    }
    if (!isVisible) {
      return (
        <tr className="table-no-rows hidden"/>
      )
    }
    let localStyle = {};
    if (this.state.left && this.state.left > 0) {
      localStyle.paddingLeft = this.state.left + 'px';
    }
    return (
      <MuiTableRow>
        <Table.Col className="table-no-rows">
          <span ref={(c) => this.row = c} style={localStyle}>
            {this.props.children}
          </span>
        </Table.Col>
      </MuiTableRow>
    )
  }
};

Table.NoRows.PropTypes = {
  if: React.PropTypes.any
};

Table.Pagination = class TablePagination extends MuiTableFooter {

  constructor(props) {
    super(props);
    if (!this.state) this.state = {};
    this.state.width = null;
    this.setWidth = this.setWidth.bind(this);
  }

  setWidth() {
    let val = this.getWidth();
    if (!val) return;
    if (val === this.state.width) return;
    this.setState({
      width: val
    });
  }

  getWidth() {
    if (!this.root) return;
    let $root = $(this.root);
    if (!$root || $root.length === 0) return;
    let $table = $root.closest('.app-table');
    if (!$table || $table.length === 0) return;
    let $tablePag = $table.find(".table-pagination");
    if ($tablePag && $tablePag.length !== 0) {
      $tablePag.parent().width($table.width());
    }
    return $table.width();
  }

  handlePage(page, type) {
    let payload = {};
    if (this.props.stripe) {
      payload.page = type;
    } else if (typeof this.props.page === 'number') {
      if (type === 'next') {
        payload.page = this.props.page + 1;
      }
      if (type === 'prev') {
        payload.page = this.props.page - 1;
      }
      if (payload.page <= 0) {
        payload.page = 1;
      }
    } else {
      if (this.filter) {
        payload = this.filter.getPayload();
      }
      payload.page = page;
    }
    if (typeof this.props.onFilter === 'function') {
      this.props.onFilter(payload);
    } else if (this.filter) {
      if (typeof this.filter.handleForm === 'function') {
        this.filter.handleForm(payload);
      } else if (typeof this.filter.set === 'function') {
        this.filter.set('page', payload.page, () => {
          this.filter.filter && this.filter.filter();
        });
      }

    }
  }

  componentDidUpdate() {
    this.setWidth();
  }

  componentDidMount() {
    if (typeof this.props.filter === 'function') {
      this.filter = this.props.filter();
    }
    this.setWidth();
    $(window).on('resize', this.setWidth);
  }

  render() {
    const {meta} = this.props;
    let footerText, prevPage = false, nextPage = false;
    if (this.props.stripe) {
      if (meta.next_page) nextPage = true;
      if (meta.prev_page) prevPage = true;
    } else if (typeof this.props.page === 'number') {
      if (this.props.page > 1) {
        prevPage = true;
      }
      if (typeof this.props.nextPage === 'boolean') {
        nextPage = this.props.nextPage;
      } else {
        nextPage = true;
      }
    } else {
      if (!meta || !meta.page_count || (meta.page_count === 1 && meta.current_page === 1)) {
        return (<MuiTableFooter/>);
      }
      footerText = <span>
        {this.props.labelShowPage}&nbsp;<b>{meta.current_page}</b>&nbsp;{this.props.labelOf}&nbsp;
        <b>{meta.page_count}</b>&nbsp;{this.props.labelTotal}&nbsp;
        <b>{meta.total_count}</b>&nbsp;{this.props.labelEntries}
       </span>;
      if (meta.current_page >= 1 && meta.current_page < meta.page_count) {
        nextPage = meta.current_page + 1;
      }
      if (meta.current_page > 1) {
        prevPage = meta.current_page - 1;
      }
    }
    return (
      <MuiTableFooter className="table-pagination">
        <MuiTableRow style={styles.height}>
          {footerText && (
            <MuiTableRowColumn style={styles.footerText}>
              {footerText}
            </MuiTableRowColumn>
          )}
          <MuiTableRowColumn style={styles.footerPaging}>
            <span ref={(c) => this.root = c}/>
            <IconButton
              disabled={prevPage == false}
              onClick={() => this.handlePage(prevPage, 'prev')}>
              <FontIcon className="pointer material-icons">chevron_left</FontIcon>
            </IconButton>
            <IconButton
              disabled={nextPage == false}
              onClick={() => this.handlePage(nextPage, 'next')}>
              <FontIcon className="pointer material-icons">chevron_right</FontIcon>
            </IconButton>
          </MuiTableRowColumn>
        </MuiTableRow>
      </MuiTableFooter>
    );
  }
};

Table.Pagination.defaultProps = {
  labelShowPage: 'Showing page',
  labelOf: 'of',
  labelTotal: 'from a total of',
  labelEntries: 'entries'
};

Table.Pagination.PropTypes = {
  labelShowPage: React.PropTypes.string,
  labelOf: React.PropTypes.string,
  labelTotal: React.PropTypes.string,
  labelEntries: React.PropTypes.string,
  onFilter: React.PropTypes.func,
  filter: React.PropTypes.any,
  page: React.PropTypes.number,
  nextPage: React.PropTypes.bool
};

const styles = {
  footerPaging: {
    float: 'right'
  },
  footerText: {
    float: 'left',
    paddingTop: 16,
    height: 16
  },
  height: {
    height: 56
  }
};


function buildStyles(props) {
  const styles = {};
  if (props.width) {
    styles.width = props.width;
    delete props.width;
  }
  if (props.align) {
    styles.textAlign = props.align;
    delete props.align;
  }
  if (props.minWidth) {
    styles.minWidth = props.minWidth;
    delete props.minWidth;
  }

  return styles;
}
