'use strict';
/**
 * Custom tab bars
 */
const React = require('react'),
  router = require('router'),
  util = require('lib/util'),
  theme = require('theme');
let tabId = 0;
import Dropdown from './Dropdown';
/*
 * PROPS:
 *   onChange=fn
 *   active=the tab name to be active.
 *   activeHref=>check the URL with each item's href attribute.
 *   position=top -> if set to bottom, render tabs below content
 * */
export default class Tabs extends React.Component {

  constructor(props) {
    super(props);
    this.checkOverflow = this.checkOverflow.bind(this);
    this._updating = false;
    this._tabItems = [];
    this.state = {
      activeTab: props.active || '',
      dropdownItems: []
    };
  }

  componentDidMount() {
    this.checkOverflow();
    $(window).on('resize', this.checkOverflow);
  }

  componentWillUnmount() {
    $(window).off('resize', this.checkOverflow);
  }

  componentDidUpdate() {
    this.checkOverflow();
  }

  /*
  * Returns all the available tab items.
  * */
  getItems() {
    let res = [];
    if (this._tabItems instanceof Array) {
      for (let i = 0, len = this._tabItems.length; i < len; i++) {
        let t = this._tabItems[i];
        if (typeof t !== 'object' || !t) continue;
        if (typeof t.props === 'object' && t.props && t.props.disabled === true) continue;
        res.push(t);
      }
    }
    return res;
  }

  checkOverflow() {
    if (this._updating) return;
    let $root = $(this.root);
    if (!$root || $root.length === 0) return;
    let overflown = [];
    $root.children().each((idx, child) => {
      let $child = $(child);
      if ($child.hasClass('app-toolbar-menu-more')) return;
      let pos = $child.position();
      if (pos.top > 25) {
        overflown.push(idx);
      }
    });
    let hasChanges = false;
    for (let i = 0; i < overflown.length; i++) {
      if (overflown[i] !== this.state.dropdownItems[i]) {
        hasChanges = true;
        break;
      }
    }
    if (!hasChanges) return;
    this._updating = true;
    this.setState({
      dropdownItems: overflown
    }, () => {
      this._updating = false;
    });
  }

  renderDropdown(items) {
    if (!items || items.length === 0) return;
    return (
      <Dropdown className="app-tabs-more" style={this.props.moreStyle}>
        {items.map((item, idx) => {
          let isActive = item.props.active;
          let itemStyle = {};
          if (isActive) {
            itemStyle.color = theme.palette.primary1Color
          }
          return (
            <Dropdown.Item
              key={idx}
              onClick={() => {
                if (item.props.disabled || item.props.inactive) return;
                item.props.onClick();
                if (item.props.href) {
                  router.navigate(item.props.href);
                }
              }}
              primaryText={item.props.title}
              style={itemStyle}
            />
          )
        })}
      </Dropdown>
    )
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.active && nextProps.active !== this.state.activeTab) {
      this.setState({
        activeTab: nextProps.active
      });
      triggerResize();
    }
  }

  render() {
    let items = [],
      overflown = [],
      children = this.props.children;
    const {dropdownItems} = this.state;
    if (!(children instanceof Array)) {
      children = [children];
    }
    let hasActive = false,
      firstIdx = 0,
      activeTabs = [],
      op = '-';
    if (this.props.activeDirection === 'left') {
      op = '+';
    }
    for (let idx = op === '-' ? children.length : 0; op === '-' ? idx >= 0 : idx < children.length; op === '-' ? idx-- : idx++) {
      let item = children[idx];
      if (typeof item !== 'object' || !item) continue;
      if (hasActive) continue;
      if (firstIdx == null) firstIdx = idx;
      let isActive = this.state.activeTab === item.props.name;
      if (!isActive && !this.state.activeTab && item.props.href) {
        if (router.containsPath(item.props.href)) {
          isActive = true;
        }
        if (!isActive && item.props.urls) {
          let urls = (item.props.urls instanceof Array ? item.props.urls : [item.props.urls]);
          for (let j = 0; j < urls.length; j++) {
            if (router.containsPath(urls[j])) {
              isActive = true;
            }
          }
        }
      }
      if (isActive) {
        hasActive = true;
        activeTabs[idx] = true;
      }
    }
    if (this.props.forceActive !== false && !hasActive && firstIdx != null) {
      activeTabs[firstIdx] = true;
    }
    let tabContent;
    if (this.props.toggle === true) tabContent = [];
    children.forEach((item, idx) => {
      if (typeof item !== 'object' || !item) return;
      if (item.type !== TabItem) {
        items.push(item);
        return;
      }
      let isActive = activeTabs[idx] || false;
      let ItemWrap = React.cloneElement(item, Object.assign({}, item.props, {
        active: isActive,
        key: item.props.name || ('tab-' + (tabId++)),
        title: item.props.title,
        onClick: () => {
          if (item.props.disabled || item.props.inactive) return;
          if (!item.props.href) {
            this.setState({
              activeTab: item.props.name
            });
          }
          this.props.onChange && this.props.onChange(item.props.name);
          item.props.onClick && item.props.onClick();
        }
      }));
      if (this.props.toggle === true) {
        tabContent.push(React.cloneElement(item.props.children, {
          key: item.props.name,
          className: isActive ? 'is-visible' : 'is-hidden'
        }));
      } else {
        if (!tabContent && isActive && item.props.children) {
          tabContent = item.props.children;
        }
      }
      items.push(ItemWrap);
      if (dropdownItems.indexOf(idx) !== -1) {
        overflown.push(ItemWrap);
      }
    });
    let position = this.props.position || 'top';
    if (tabContent) {
      tabContent = <div className="app-tabs-content" style={this.props.contentStyle}>
        {tabContent}
      </div>;
    }
    this._tabItems = items;
    return (
      <div className={classNames(this.props.className, {
        'is-overflown': overflown.length > 0
      }, 'app-tabs', 'app-tabs-' + position)} style={this.props.style}>
        {position === "top" ?
          <div style={this.props.titleStyle} className="app-tabs-title" ref={(c) => this.root = c}>
            {items}
          </div> : tabContent
        }
        {position === "top" ?
          tabContent :
          <div style={this.props.titleStyle} className="app-tabs-title" ref={(c) => this.root = c}>
            {items}
          </div>
        }
        {this.renderDropdown(overflown)}
      </div>
    )
  }
}

Tabs.PropTypes = {
  active: React.PropTypes.any,
  activeDirection: React.PropTypes.string,  // loop direction (right or left)
  name: React.PropTypes.any,
  href: React.PropTypes.any,
  urls: React.PropTypes.any,
  toggle: React.PropTypes.any,
  title: React.PropTypes.any,
  onChange: React.PropTypes.func,
  onClick: React.PropTypes.func,
  position: React.PropTypes.func,
  moreStyle: React.PropTypes.func,
  contentStyle: React.PropTypes.any,
  titleStyle: React.PropTypes.any
};

/*
 * PROPS
 *   title= the title of the tabs.
 *   onClick=once the tab was clicked.
 *   name=the unique name of the tab.
 *   href=url to go to
 * */
class TabItem extends React.Component {

  render() {
    let subView = this.props.title;
    if (this.props.href) {
      if (this.props.inactive === true || this.props.disabled === true) {
        subView = this.props.title;
      } else {
        subView = <a href={this.props.href}>{this.props.title}</a>
      }
    } else {
      subView = <a>{this.props.title}</a>;
    }

    return (
      <div
        onClick={(e) => {
          if (this.props.inactive === true || this.props.disabled === true) {
            e.preventDefault();
            e.stopPropagation();
            return false;
          }
          triggerResize();
          this.props.onClick && this.props.onClick();
        }}
        style={this.props.style}
        className={classNames(this.props.className, {
          'app-tabs-item': true,
          'active': this.props.active,
          'inactive': this.props.inactive || this.props.disabled
        })}>
        {subView}
      </div>
    )
  }

}

Tabs.Item = TabItem;
TabItem.PropTypes = {
  title: React.PropTypes.any,
  href: React.PropTypes.any,
  inactive: React.PropTypes.any,
  disabled: React.PropTypes.any,
  onClick: React.PropTypes.func,
  active: React.PropTypes.any
};

function triggerResize() {
  setTimeout(() => {
    util.triggerResize();
  }, 5);
}
