'use strict';
/**
 * Created by Adrian on 15-Sep-16.
 */
const React = require('react');
import TextField from 'material-ui/TextField';
import ActionVisibility from 'material-ui/svg-icons/action/visibility';
import ActionVisibilityOff from 'material-ui/svg-icons/action/visibility-off';

let id = 0;

export default class AppTextField extends React.Component {

  constructor(props) {
    super(props);
    this.handleChange = this.handleChange.bind(this);
    this.state = {};
    if (props.maxLength && props.maxLengthShow) {
      let val = props.value || props.defaultValue || '';
      let remaining = this.props.maxLength - val.length;
      this.state.remaining = remaining;
    }
    if (props.type === 'password') {
      this.state.passwordVisible = false;
    }
    this._timer = null;
  };

  componentWillReceiveProps(nextProps) {
    if (nextProps.maxLength && nextProps.maxLengthShow) {
      let val = this.getValue();
      if (val) {
        let remaining = nextProps.maxLength - val.length;
        this.state.remaining = remaining;
      }
    }
  }

  getValue() {
    return this.refs.textField && this.refs.textField.getValue();
  }

  setValue(v, _triggerChange) {
    if (!this.refs.textField) return;
    if (this.props.rowsMax || this.props.multiLine || this.props.rows) {
      try {
        this.refs.textField.input.refs.input.value = v;
      } catch (e) {
        console.log(e);
      }
    } else {
      this.refs.textField.input.value = v;
    }
    this.triggerChange();
    if (!v) {
      this.refs.textField.setState({
        hasValue: false
      });
    } else if (!this.refs.textField.state.hasValue) {
      this.refs.textField.setState({
        hasValue: true
      });
    }
    if (_triggerChange === true) {
      this.handleChange(null, v, true);
    }
  }

  componentWillUnmount() {
    if (this.props.delayed !== true) return;
    if (this._timer == null) return;
    clearTimeout(this._timer);
    this._timer = null;
    this.props.onChange && this.props.onChange(this._changeVal);
  }

  triggerChange() {
    $(this.refs.textField.input).trigger('change');
  }

  handleChange(e, val, _forceChange) {
    if (this.props.maxLength > 1 && this.maxLengthShow) {
      let remaining = this.props.maxLength - val.length;
      this.setState({
        remaining
      });
    }
    if (this.props.delayed === true) {
      clearTimeout(this._timer);
      this._changeVal = val;
      if (_forceChange !== true) {
        this._timer = setTimeout(() => {
          this.props.onChange && this.props.onChange(val);
          this._timer = null;
        }, this.props.delayedTime);
        return;
      }
    }
    this.props.onChange && this.props.onChange(val);
  }

  focus() {
    this.refs.textField && this.refs.textField.focus();
  }

  blur() {
    this.refs.textField && this.refs.textField.blur();
  }

  render() {
    let props = Object.assign({}, {
      id: 'field-' + (id++),
      fullWidth: true,
      floatingLabelText: this.props.title
    }, this.props);
    delete props.title;
    delete props.delayed;
    if (typeof props.maxLength === 'number' && props.maxLengthShow !== false) {
      props.floatingLabelText += ` ${(props.maxLength - this.state.remaining) || 0} / ${props.maxLength}`;
    }
    if (typeof props.errorText === 'object' && props.errorText) {
      props.errorText = props.errorText[props.name] || '';
    }
    if (typeof props.defaultValue === 'object' && props.defaultValue) {
      props.defaultValue = props.defaultValue[props.name] || '';
    }
    if (typeof props.value === 'object' && props.value) {
      props.value = props.value[props.name];
      if (!props.value) delete props.value;
    }
    if (props.reset === true && !props.key) {
      props.key = Date.now();
      delete props.reset;
    }
    if (this.props.maxLengthShow) {
      this.maxLengthShow = true;
    }
    delete props.maxLengthShow;
    delete props.delayedTime;
    let togglePassword;
    if (props.type === 'password') {
      if (this.state.passwordVisible === true) {
        props.type = 'text';
      } else {
        props.type = 'password';
      }
      let icon;
      if (typeof this.props.renderPasswordIcon === 'function') {
        icon = this.props.renderPasswordIcon(this.state.passwordVisible);
      } else {
        icon = (this.state.passwordVisible ? <ActionVisibilityOff/> : <ActionVisibility/>);
      }
      togglePassword = (
        <span
          className="app-textfield-password-toggle"
          onClick={() => this.setState({
            passwordVisible: !this.state.passwordVisible
          })}
        >
          {icon}
        </span>
      );
    }
    delete props.renderPasswordIcon;
    if (this.props.delayed) {
      let _old = props.onKeyPress;
      props.onKeyPress = (e) => {
        if (e.charCode === 13) {
          let val = this.getValue();
          this.handleChange(e, val, true);
        }
        _old && _old(e);
      }
    }
    if (typeof this.props.rowsMax === 'number' && typeof this.props.multiLine !== 'boolean') {
      props.multiLine = true;
    }
    let textView = <TextField
      {...props}
      ref="textField"
      onChange={this.handleChange}
      hintStyle={{fontSize: 12}}
      errorStyle={{marginTop: 24}}
      className={classNames("app-textfield", {
        'is-multiline': props.multiLine,
        'has-label': !!props.floatingLabelText
      }, props.className)}
    />;
    if (togglePassword) {
      return (
        <div style={{position: 'relative'}}>
          {textView}
          {togglePassword}
        </div>
      )
    }
    return textView;
  }

};
AppTextField.defaultProps = {
  maxLengthShow: true,
  delayedTime: 200
};
AppTextField.PropTypes = {
  name: React.PropTypes.string,
  maxLength: React.PropTypes.any,
  maxLengthShow: React.PropTypes.any,
  value: React.PropTypes.any,
  defaultValue: React.PropTypes.any,
  delayed: React.PropTypes.any,
  delayedTime: React.PropTypes.number,
  onChange: React.PropTypes.func,
  title: React.PropTypes.any,
  renderPasswordIcon: React.PropTypes.func,
  floatingLabelText: React.PropTypes.any,
  errorText: React.PropTypes.any,
  reset: React.PropTypes.any,
  key: React.PropTypes.any,
};
