'use strict';

const React = require('react'),
  router = require('router'),
  tredux = require('tredux'),
  theme = require('theme');

import Dropdown from './Dropdown';

export default class ToolbarMenu extends React.Component {

  constructor(props) {
    super(props);
    this.checkOverflow = this.checkOverflow.bind(this);
    this.state = {
      dropdownItems: []
    };
    this._updating = false;
    this._menuItems = [];
  }

  componentDidUpdate() {
    this.checkOverflow();
  }

  componentDidMount() {
    this.checkOverflow();
    $(window).on('resize', this.checkOverflow);
  }

  componentWillUnmount() {
    $(window).off('resize', this.checkOverflow);
  }

  getItems() {
    let res = [];
    if (this._menuItems instanceof Array) {
      for (let i = 0, len = this._menuItems.length; i < len; i++) {
        let p = this._menuItems[i];
        if (typeof p !== 'object' || !p) continue;
        if (typeof p.props === 'object' && p.props && p.props.disabled === true) continue;
        res.push(p);
      }
    }
    return res;
  }

  checkOverflow() {
    if (this._updating) return;
    let $root = $(this.root);
    if (!$root || $root.length === 0) return;
    let overflown = [];
    $root.children().each((idx, child) => {
      let $child = $(child);
      if ($child.hasClass('app-toolbar-menu-more')) return;
      let pos = $child.position();
      if (pos.top !== 0) {
        overflown.push(idx);
      }
    });
    let hasChanges = false;
    for (let i = 0; i < overflown.length; i++) {
      if (overflown[i] !== this.state.dropdownItems[i]) {
        hasChanges = true;
        break;
      }
    }
    if (!hasChanges) return;
    this._updating = true;
    this.setState({
      dropdownItems: overflown
    }, () => {
      this._updating = false;
    });
  }


  renderDropdown(items) {
    if (!items || items.length === 0) return;
    return (
      <Dropdown className="app-toolbar-menu-more">
        {items.map((item, idx) => {
          let isActive = (typeof item.props.active === 'boolean' ?
            item.props.active :
            router.containsPath(item.props.href))
          let itemStyle = {};
          if (isActive) {
            itemStyle.color = theme.palette.primary1Color
          }
          return (
            <Dropdown.Item
              key={idx}
              href={item.props.href}
              primaryText={item.props.title}
              style={itemStyle}
            />
          )
        })}
      </Dropdown>
    )
  }

  render() {
    const {dropdownItems} = this.state;
    let children = [],
      result = [],
      items = [];
    if (this.props.children instanceof Array) {
      children = this.props.children;
    } else if (typeof this.props.children === 'object' && this.props.children) {
      children = [this.props.children];
    }
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (typeof  child !== 'object' || !child) continue;
      result.push(child);
      if (dropdownItems.indexOf(i) !== -1) {
        items.push(child);
      }
    }
    this._menuItems = result;
    return (
      <div ref={(c) => this.root = c} className="app-toolbar-menu">
        {result}
        {this.renderDropdown(items)}
      </div>
    )
  }
}

/*
* PROPS
*   title
*   href
* */
ToolbarMenu.Item = class ToolbarMenuItem extends React.Component {
  render() {
    let isActive = router.containsPath(this.props.href);
    if (typeof this.props.active === 'boolean') {
      isActive = this.props.active;
    }
    return (
      <div className={classNames('app-toolbar-menu-item', {
        'active': isActive
      })}>
        <a href={this.props.href}>
          {this.props.title}
        </a>
      </div>
    )
  }
};
ToolbarMenu.Item.PropTypes = {
  title: React.PropTypes.any,
  href: React.PropTypes.any,
  active: React.PropTypes.any
};
