'use strict';
/**
 * Main console app entry point
 */
const React = require('react'),
  tredux = require('tredux'),
  router = require('router');
let Menu = require('root/containers/Menu'),
  Submenu = require('root/containers/Submenu'),
  Header = require('root/containers/Header'),
  Theme = require('./Theme'),
  AppContent = require('../components/AppContent'),
  AppToast = require('../components/AppToast'),
  Loading = require('../containers/Loading'),
  Confirmation = require('../containers/Confirmation');

if (typeof Submenu !== 'function') Submenu = null;
if (typeof Menu !== 'function') Menu = null;
if (typeof Header !== 'function') Header = null;

module.exports = class App extends React.Component {

  getTheme(children) {
    return (
      <Theme>{children}</Theme>
    )
  }

  getContent(children) {
    return (
      <AppContent>{children}</AppContent>
    );
  }

  getRoot(children) {
    return (
      <div className="app-theme">
        {children}
      </div>
    )
  }

  render() {
    return tredux.mount(
      this.getTheme(
        this.getRoot(
          <div>
            <Loading/>
            <Confirmation/>
            {Header && <Header/>}
            {Menu && <Menu/>}
            {Submenu && <Submenu/>}
            {this.getContent(router.mount())}
            <AppToast/>
          </div>
        )
      )
    );

    return tredux.mount(
      <Theme>
        <div className="app-theme">
          <Loading/>
          <Confirmation/>
          {Header && <Header/>}
          {Menu && <Menu/>}
          {Submenu && <Submenu/>}
          <AppContent>
            {router.mount()}
          </AppContent>
          <AppToast/>
        </div>
      </Theme>
    );
  }
};
