'use strict';

const React = require('react'),
  tredux = require('tredux'),
  api = require('api'),
  router = require('router');

import Dialog from 'core/components/Dialog';

class AppConfirmation extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      active: false
    };
    this.hide = this.hide.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.handleClose = this.handleClose.bind(this);
    this.handleEscape = this.handleEscape.bind(this);
  }

  hide() {
    this.setState({
      active: false,
      auto_hide: true
    });
  }

  handleDelete() {
    let fn = this.state.fn;
    let res = fn(true, this.hide);
    if (this.state.auto_hide === false) return;
    if (typeof res === 'object' && typeof res.then === 'function' && typeof res.catch === 'function') {
      res.then(() => this.hide()).catch((e) => {
        console.warn(e);
      });
    } else {
      this.hide();
    }
  }

  handleClose() {
    let fn = this.state.fn;
    fn(false);
    this.hide();
  }

  handleEscape(e) {
    if (e.keyCode !== 27) return;
    if (!this.state.fn) return;
    this.handleClose();
  }

  componentDidMount() {
    this.listener = tredux.addListener('CONFIRMATION', (info) => {
      info.active = true;
      Object.keys(this.state).forEach((name) => {
        if (typeof info[name] === 'undefined') {
          info[name] = null;
        }
      });
      this.setState(info);
      if(typeof info.close === 'function') {
        info.close(this.handleClose);
      }
    });
    document.addEventListener("keydown", this.handleEscape, false);
  }

  componentWillUnmount() {
    document.removeEventListener("keydown", this.handleEscape, false);
    this.listener.remove();
  }

  render() {
    if (!this.state.active) return (<div/>);
    const data = this.state;
    let actions = [];
    if (this.state.buttons instanceof Array) {
      actions = this.state.buttons;
    } else {
      actions.push(<Dialog.Cancel
        label={data.cancel || 'Cancel'}
        onTouchTap={this.handleClose}
      />);
      if (data.button !== false) {
        let names = 'align-left';
        if (data.is_delete !== false) {
          names += ' bg-delete';
        }
        actions.push(
          <Dialog.Button
            className={names}
            label={data.button || 'Delete'}
            onClick={this.handleDelete}
          />
        );
      }
    }
    let msg;
    if (typeof data.message === 'string') {
      msg = <p>{data.message}</p>;
    } else if (typeof data.render === 'function') {
      msg = data.render();
    } else if (typeof data.content === 'object' && data.content) {
      msg = data.content;
    } else {
      msg = <p>Are you sure?</p>;
    }
    let style = {
      maxWidth: 512
    };
    if (this.state.style) {
      style = this.state.style;
    }
    return (
      <Dialog
        title={data.title || 'Confirmation'}
        actions={actions}
        style={style}>
        {msg}
      </Dialog>
    )
  }

}

module.exports = AppConfirmation;
