'use strict';
/**
 * This is the base App Menu that we can extend when rendering it.
 * NOTE:
 * this MUST be connected to tredux for the "core.ui" state.
 * */
const React = require('react'),
  tredux = require('tredux'),
  AppMenu = require('../components/AppMenu'),
  AppMenuItem = require('../components/AppMenuItem');

export default class Menu extends React.Component {
  componentDidMount() {
    this.setPadding();
    $(window).on('scroll', this.setPadding);
  }

  componentWillUnmount() {
    $(window).off('scroll', this.setPadding);
  }

  setPadding() {
    let $header = $("#appHeader");
    if (!$header) {
      setTimeout(() => {
        this.setPadding();
      }, 10);
      return;
    }
    let scroll = $(window).scrollTop(),
      diff = Math.max(64 - scroll, 64);
    if (this.paddingTop === diff) return;
    this.paddingTop = diff;
    $("#appMenu").css("paddingTop", diff);
  }

  /*
  * Returns an array of menu items to render.
  * */
  getMenuItems() {
    return [];
  }

  /*
  * Returns the react components for the footer menu when opened
  * */
  getFooter() {
  }

  /*
  * Returns the react component for when menu is closed.
  * */
  getFooterIcon() {
  }


  render() {
    const items = this.getMenuItems(),
      footer = this.getFooter(),
      footerIcon = this.getFooterIcon();
    return (
      <AppMenu
        id="appMenu"
        footer={footer}
        footerIcon={footerIcon}
        onSubmenuShow={(subMenu) => {
          const ui = tredux.getState()['core.ui'] || {};
          if (ui.subMenu !== subMenu) {
            tredux.dispatch(tredux.actions('core.ui').minimizeMenu(subMenu))
          }
        }}
        onSubmenuHide={() => {
          const ui = tredux.getState()['core.ui'] || {};
          if (ui.menuMinimized) {
            tredux.dispatch(tredux.actions('core.ui').maximizeMenu())
          }
        }}
      >
        {items}
      </AppMenu>
    );
  }
}

Menu.Item = AppMenuItem;



