'use strict';
/**
 * UI specific
 */
const tredux = require('tredux'),
  config = require('config'),
  TYPE = require('../actions/ui').TYPE;

const MENU_OPEN_KEY = 'umenuopened';

function getMenuOpen() {
  try {
    return localStorage.getItem(MENU_OPEN_KEY) === "true";
  } catch (e) {
    return false;
  }
}

function setMenuOpen(v) {
  try {
    localStorage.setItem(MENU_OPEN_KEY, v.toString());
  } catch (e) {
  }
}

const reducer = tredux.reducer('core.ui', {
  booted: false,
  title: '',
  url: {
    path: window.location.href.split('#')[0],
    payload: {}
  },
  menuOpened: getMenuOpen(),
  menuMinimized: false,
  subMenu: null
});

reducer.handle(TYPE.IS_BOOTED, (state) => {
  state.booted = true;
});

reducer.handle(TYPE.URL_CHANGE, (state, payload) => {
  state.url = {
    path: payload.url,
    payload: payload.payload || {}
  };
});

reducer.handle(TYPE.SHOW_MENU, (state) => {
  setMenuOpen(true);
  state.menuOpened = true;
  state.menuMinimized = false;
  state.subMenu = null;
});

reducer.handle(TYPE.HIDE_MENU, (state) => {
  setMenuOpen(false);
  state.menuOpened = false;
  state.menuMinimized = false;
  state.subMenu = null;
});
reducer.handle(TYPE.MINIMIZE_MENU, (state, payload) => {
  state.menuMinimized = payload.active;
  state.subMenu = typeof payload.subMenu === 'string' ? payload.subMenu : null;
});


reducer.handle(TYPE.SET_TITLE, (state, payload) => {
  state.title = payload.title;
});
