'use strict';
const path = require('path');
/**
 * This contains the default webpack configuration
 * that must be included by the root application webpack,
 * in order to include all our entries, aliases,etc.
 * */
const config = {};

module.exports = (config, rootDir) => {
  if (typeof config !== 'object' || !config) config = {};
  let finalConfig = {
    entry: [
      path.normalize(__dirname + '/src/lib/shim'),
      path.normalize(__dirname + '/src/modules/core/index'),
      path.normalize(__dirname + '/src/styles/index.less')
    ],
    output: {
      path: path.normalize(rootDir + '/public/js'),
      library: "ui",
      publicPath: '/',
      filename: "ui.js"
    },
    module: {
      loaders: []
    },
    plugins: [],
    resolve: {
      alias: {
        "React": "react",
        "react": "react",
        "uheader": path.normalize(__dirname + "/src/lib/uheader"),
        "tredux-material/countries": path.normalize(__dirname + "/src/config/countries"),
        "tredux-material/api": path.normalize(__dirname + "/src/lib/api"),
        "tredux-material/util": path.normalize(__dirname + "/src/lib/util"),
        "modules/core": path.normalize(__dirname + "/src/modules/core"),
        "theme": path.normalize(__dirname + "/src/config/theme"),
        "root": path.normalize(rootDir + '/src/modules/root'),
        "config": path.normalize(rootDir + '/src/config/' + process.env.NODE_ENV),
        "config/countries": path.normalize(__dirname + "/src/config/countries"),
        "core": path.normalize(__dirname + "/src/modules/core"),
        "router": path.normalize(__dirname + "/src/lib/router"),
      },
      root: [
        path.normalize(rootDir + "/src/lib"),
        path.normalize(rootDir + "/src/modules"),
        path.normalize(rootDir + '/node_modules'),
        path.normalize(__dirname + '/node_modules'),
        path.normalize(__dirname + '/src/lib'),
        path.normalize(__dirname + '/src/modules/core')
      ],
      modulesDirectories: ['node_modules', 'src'],
      extensions: ['', '.js', '.jsx']
    },
    externals: {}
  };
  let baseKeys = Object.keys(finalConfig);

  if (config.entry instanceof Array) {
    copy(finalConfig.entry, config.entry);
  }
  if (config.output) {
    copy(finalConfig.output, config.output);
  }
  if (config.module) {
    finalConfig.module = config.module;
  }
  if (config.plugins) {
    finalConfig.plugins = config.plugins;
  }
  if (config.resolve) {
    if (config.resolve.alias) {
      copy(finalConfig.resolve.alias, config.resolve.alias);
    }
    if (config.resolve.root) {
      copy(finalConfig.resolve.root, config.resolve.root);
    }
    if (config.resolve.modulesDirectories) {
      copy(finalConfig.resolve.modulesDirectories, config.resolve.modulesDirectories);
    }
    if (config.resolve.extensions) {
      copy(finalConfig.resolve.extensions, config.resolve.extensions);
    }
  }
  if (config.externals) {
    copy(finalConfig.externals, config.externals);
  }
  Object.keys(config).forEach((name) => {
    if (baseKeys.indexOf(name) !== -1) return;
    finalConfig[name] = config[name];
  });
  return finalConfig;
};


function copy(target, source) {
  if (source instanceof Array && target instanceof Array) {
    for (let i = 0, len = source.length; i < len; i++) {
      target.push(source[i]);
    }
    return;
  }
  Object.keys(source).forEach((name) => {
    target[name] = source[name];
  });
}
